#!/usr/bin/env python3


#% $Id$ 

#
# Copyright (C) 2002-2007
# ASTRON (Netherlands Foundation for Research in Astronomy)
# and The MeqTree Foundation
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#  (c) 2013.				 (c) 2011.
#  National Research Council		 Conseil national de recherches
#  Ottawa, Canada, K1A 0R6 		 Ottawa, Canada, K1A 0R6
#
#  This software is free software;	 Ce logiciel est libre, vous
#  you can redistribute it and/or	 pouvez le redistribuer et/ou le
#  modify it under the terms of	         modifier selon les termes de la
#  the GNU General Public License	 Licence Publique Generale GNU
#  as published by the Free		 publiee par la Free Software
#  Software Foundation; either	 	 Foundation (version 3 ou bien
#  version 2 of the License, or	 	 toute autre version ulterieure
#  (at your option) any later	 	 choisie par vous).
#  version.
#
#  This software is distributed in	 Ce logiciel est distribue car
#  the hope that it will be		 potentiellement utile, mais
#  useful, but WITHOUT ANY		 SANS AUCUNE GARANTIE, ni
#  WARRANTY; without even the	 	 explicite ni implicite, y
#  implied warranty of			 compris les garanties de
#  MERCHANTABILITY or FITNESS FOR	 commercialisation ou
#  A PARTICULAR PURPOSE.  See the	 d'adaptation dans un but
#  GNU General Public License for	 specifique. Reportez-vous a la
#  more details.			 Licence Publique Generale GNU
#  					 pour plus de details.
#
#  You should have received a copy	 Vous devez avoir recu une copie
#  of the GNU General Public		 de la Licence Publique Generale
#  License along with this		 GNU en meme temps que ce
#  software; if not, contact the	 logiciel ; si ce n'est pas le
#  Free Software Foundation, Inc.	 cas, communiquez avec la Free
#  at http://www.fsf.org.		 Software Foundation, Inc. au
#						 http://www.fsf.org.
#
#  email:				 courriel:
#  business@hia-iha.nrc-cnrc.gc.ca	 business@hia-iha.nrc-cnrc.gc.ca
#
#  National Research Council		 Conseil national de recherches
#      Canada				    Canada
#  Herzberg Institute of Astrophysics	 Institut Herzberg d'astrophysique
#  5071 West Saanich Rd.		 5071 West Saanich Rd.
#  Victoria BC V9E 2E7			 Victoria BC V9E 2E7
#  CANADA					 CANADA
#
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

import sys
import numpy
HAS_TIMBA = False
try:
  from Timba.utils import verbosity
  _dbg = verbosity(0,name='SpectrumData');
  _dprint = _dbg.dprint;
  _dprintf = _dbg.dprintf;
  HAS_TIMBA = True
except:
  pass

class SpectrumData:
   """ a class to store spectral data and supply the
       data to the browser display """

   def __init__(self, data_label='', string_tag=''):
     self._data_labels = data_label
     self._string_tag = string_tag
     self._active_plot = 0
     self.num_y_markers = 0
     self.y_marker_step = 0

     self._plot_dict = {}
     self._plot_labels = {}
     self._menu_labels = {}
     self._combined_label_dict = {}
     self.marker_labels = []

#    self.__init__


   def StoreSpectrumData(self, incoming_data):
     """ This method does most of the work. It stores the data,
         creates labels for the browser pull-down menu, and
         also creates titles for the plots """

     self._plot_dict = {}
     self._plot_labels = {}
     self._menu_labels = {}
     self._combined_label_dict = {}
     self.marker_labels = []

     num_plot_arrays = len(incoming_data)
     if HAS_TIMBA:
       _dprint(2,' number of arrays to plot ', num_plot_arrays)
     for i in range(num_plot_arrays):
       menu_label = ''
       plot_label = ''
       combined_display_label = ''
       if isinstance(self._data_labels, tuple):
         menu_label = 'go to ' + self._string_tag  +  " " +self._data_labels[i] 
         combined_display_label = self._string_tag  +  " " + self._data_labels[i]
         plot_label = 'spectra:' + combined_display_label
       else:
         menu_label = 'go to ' + self._string_tag  +  " " +self._data_labels 
         combined_display_label = self._string_tag  +  " " + self._data_labels
         plot_label = 'spectra:' + combined_display_label

# flip plot array for screen display
       axes = numpy.arange(incoming_data[i].ndim)[::-1]
       plot_array = numpy.transpose(incoming_data[i], axes)

# has plot array been stored?
       if len(self._plot_labels) == 0:
         self._menu_labels[0] = menu_label
         self._plot_labels[0] = plot_label
         self._combined_label_dict[0] = combined_display_label
         self._plot_dict[0] = plot_array
        
       else:
         plot_found = False
         if HAS_TIMBA:
           _dprint(2,' plot_labels ', self._plot_labels)
         for j in range(len(self._plot_labels)):
           if self._plot_labels[j] == plot_label:
             self._plot_dict[j] = plot_array
             plot_found = True
             break
         if not plot_found:
           index = len(self._plot_labels)
           self._plot_dict[index] = plot_array
           self._combined_label_dict[index] = combined_display_label
           self._plot_labels[index] = plot_label
           self._menu_labels[index] = menu_label

# create combined image?
     if len(self._plot_dict) > 1:
       plot_dict_size = len(self._plot_dict)
       if HAS_TIMBA:
         _dprint(3,' creating combined array')
       shape = self._plot_dict[0].shape
       self.y_marker_step = shape[1]
       self.num_y_markers = plot_dict_size
       if HAS_TIMBA:
         _dprint(3,' self.y_marker_step ', self.y_marker_step)
       temp_array = numpy.zeros((shape[0], plot_dict_size * shape[1]),dtype=self._plot_dict[0].dtype)
       for i in range(plot_dict_size):
         dummy_array =  self._plot_dict[i]
         for j in range(shape[0]):
           for k in range(shape[1]):
             k_index = i * shape[1] + k
             temp_array[j,k_index] = dummy_array[j,k]
         self.marker_labels.append(self._combined_label_dict[i])
       self._plot_dict[plot_dict_size] = temp_array
       self._plot_labels[plot_dict_size] = 'spectra: combined image'
       self._menu_labels[plot_dict_size] = 'go to combined image'

   def getMenuLabels(self):
     return self._menu_labels

   def getPlotLabels(self):
     return self._plot_labels

   def getPlotLabel(self):
     return self._plot_labels[self._active_plot]

   def getPlotData(self):
     return self._plot_dict

   def getPlotDictSize(self):
     return len(self._plot_dict)

   def getActivePlot(self):
     return self._active_plot

   def setActivePlot(self, active_plot=0):
     self._active_plot = active_plot

   def getActivePlotArray(self):
     return self._plot_dict[self._active_plot]

   def getMarkerParms(self):
     return [self.num_y_markers, self.y_marker_step] 

   def getMarkerLabels(self):
     return self.marker_labels

def main(args):
  print('we are in main')

# Admire
if __name__ == '__main__':
    main(sys.argv)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# The QwtPlotImage class is just a straightforward adaption of 
# the same class in PyQwt-4.2/examples/QwtImagePlotDemo.py.
# The class has been modified to work with the color maps
# used for MeqTrees and also to display flagged arrays.
# The coordinate functions have also been modified to give
# increased accuracy for zooming, etc.


#% $Id: QwtPlotImage.py 6836 2009-03-05 18:55:17Z twillis $ 

#
# Copyright (C) 2002-2007
# ASTRON (Netherlands Foundation for Research in Astronomy)
# and The MeqTree Foundation
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#  (c) 2013.				 (c) 2011.
#  National Research Council		 Conseil national de recherches
#  Ottawa, Canada, K1A 0R6 		 Ottawa, Canada, K1A 0R6
#
#  This software is free software;	 Ce logiciel est libre, vous
#  you can redistribute it and/or	 pouvez le redistribuer et/ou le
#  modify it under the terms of	         modifier selon les termes de la
#  the GNU General Public License	 Licence Publique Generale GNU
#  as published by the Free		 publiee par la Free Software
#  Software Foundation; either	 	 Foundation (version 3 ou bien
#  version 2 of the License, or	 	 toute autre version ulterieure
#  (at your option) any later	 	 choisie par vous).
#  version.
#
#  This software is distributed in	 Ce logiciel est distribue car
#  the hope that it will be		 potentiellement utile, mais
#  useful, but WITHOUT ANY		 SANS AUCUNE GARANTIE, ni
#  WARRANTY; without even the	 	 explicite ni implicite, y
#  implied warranty of			 compris les garanties de
#  MERCHANTABILITY or FITNESS FOR	 commercialisation ou
#  A PARTICULAR PURPOSE.  See the	 d'adaptation dans un but
#  GNU General Public License for	 specifique. Reportez-vous a la
#  more details.			 Licence Publique Generale GNU
#  					 pour plus de details.
#
#  You should have received a copy	 Vous devez avoir recu une copie
#  of the GNU General Public		 de la Licence Publique Generale
#  License along with this		 GNU en meme temps que ce
#  software; if not, contact the	 logiciel ; si ce n'est pas le
#  Free Software Foundation, Inc.	 cas, communiquez avec la Free
#  at http://www.fsf.org.		 Software Foundation, Inc. au
#						 http://www.fsf.org.
#
#  email:				 courriel:
#  business@hia-iha.nrc-cnrc.gc.ca	 business@hia-iha.nrc-cnrc.gc.ca
#
#  National Research Council		 Conseil national de recherches
#      Canada				    Canada
#  Herzberg Institute of Astrophysics	 Institut Herzberg d'astrophysique
#  5071 West Saanich Rd.		 5071 West Saanich Rd.
#  Victoria BC V9E 2E7			 Victoria BC V9E 2E7
#  CANADA					 CANADA
#
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

import sys
import numpy
import math

from qwt.qt.QtGui import QApplication, QPen, QWidget, QFrame, QImage, qRgb
from qwt.qt.QtCore import Qt, QSize, QObject
from qwt import (QwtPlot, QwtPlotMarker, QwtPlotGrid, QwtPlotCurve,
                 QwtPlotItem, QwtText, QwtLinearColorMap)

HAS_TIMBA = False

try:
  from MeqGUI.Plugins.ImageScaler import ImageScaler
  from MeqGUI.Plugins.QwtSpy_qt5 import Spy
  from Timba.utils import verbosity
  dbg = verbosity(0,name='QwtPlotImage');
  dprint = _dbg.dprint;
  dprintf = _dbg.dprintf;
  HAS_TIMBA = True
except:
  pass


# from scipy.pilutil
# note: low is set to 2, so that we can save a value of 0 for a flagged pixel
# and 1 for a pixel with Nans or Infs
def bytescale(data, limits, high=255, low=2):
    if data.dtype == numpy.uint8:
        return data
    high = high - low
    if limits[0] is None:
        limits[0] = data.min()
    if limits[1] is None:
        limits[1] = data.max()
    scale = high *1.0 / (limits[1]-limits[0] or 1)
    internal_data = data.copy()
    temp1 = numpy.less_equal(data,limits[1])
    temp2 = numpy.greater(data,limits[1])
    internal_data = data * temp1 + temp2 * limits[1]
    temp1 = numpy.greater_equal(internal_data,limits[0])
    temp2 = numpy.less(internal_data,limits[0])
    internal_data = internal_data * temp1 + temp2 * limits[0]
    bytedata = ((internal_data*1.0-limits[0])*scale + 0.4999).astype(numpy.uint8) + numpy.asarray(low).astype(numpy.uint8)
    return bytedata

def sinx_image(nx,ny,off=0):
# creates a test image as just a scaled sin(x) / x. 
# One could probably compute this more quickly.
    num_ys = ny
    num_xs = nx
    image_numpy = numpy.ones((num_ys,num_xs),dtype=numpy.float32)
    for k in range(num_ys):
      k_dist = abs (k - num_ys/2 - off )
      for i in range(num_xs):
        i_dist = abs (i - num_xs/2 - off)
        dist = math.sqrt(k_dist*k_dist + i_dist*i_dist)
        if dist != 0:
          image_numpy[k,i] =  math.sin(dist) / dist
    return image_numpy
# sinx_image

# called by the QwtImagePlot class
def square(n, min, max):
    t = numpy.arange(min, max, float(max-min)/(n-1))
    #return outer(cos(t), sin(t))
    return numpy.cos(t)*numpy.sin(t)[:,numpy.newaxis]
# square()

def newToQImage(array):
    """Converts a numpy array to a QImage 
    A Python version of PyQt4.Qwt5.toQImage(array) in PyQwt >= 5.2.
    Function written by Gerard Vermeulen
    """
    if array.ndim != 2:
        raise RuntimeError('array must be 2-D')
    height, width = array.shape
    if array.dtype == numpy.uint8:
        # image = QImage(array, width, height, QImage.Format_Indexed8)
        # The next statement shows that QImage does not increase the
        # reference count of the buffer object to keep the data valid.
        image = Qt.QImage(
            array.tostring(), width, height, Qt.QImage.Format_Indexed8)
        image.setNumColors(256)
        for i in range(256):
            image.setColor(i, qRgb(i, i, i))
        return image
    elif array.dtype == numpy.uint32:
        image = Qt.QImage(
            array.tostring(), width, height, Qt.QImage.Format_ARGB32)
        return image
    else:
        raise RuntimeError('array.dtype must be uint8 or uint32')
# newToQImage()

def oldToQImage(array):
    """Converts a numpy array to a QImage 
    A Python version of PyQt4.Qwt5.toQImage(array) in PyQwt < 5.2.
    Function written by Gerard Vermeulen 
    """
    if array.ndim != 2:
        raise RuntimeError('array must be 2-D')
    nx, ny = array.shape # width, height
    xstride, ystride = array.strides
    if array.dtype == numpy.uint8:
        image = QImage(nx, ny, QImage.Format_Indexed8)
        f_array = numpy.reshape(array,(nx*ny,),order='F')
        for j in range(ny):
            pointer = image.scanLine(j)
            pointer.setsize(nx*array.itemsize)
            memory = numpy.frombuffer(pointer, numpy.uint8)
            first_value = j*nx
            last_value = (j+1)*nx 
            memory[:] = f_array[first_value:last_value]
        image.setColorCount(256)
        for i in range(256):
            image.setColor(i, qRgb(i, i, i))
        return image
    elif array.dtype == numpy.uint32:
        image = Qt.QImage(
            array.tostring(), width, height, Qt.QImage.Format_ARGB32)
        f_array = numpy.reshape(array,(nx*ny,),order='F')
        for j in xrange(ny):
            pointer = image.scanLine(j)
            pointer.setsize(nx*array.itemsize)
            memory = numpy.frombuffer(pointer, numpy.uint32)
            first_value = j*nx
            last_value = (j+1)*nx 
            memory[:] = f_array[first_value:last_value]
        return image
    else:
        raise RuntimeError('array.dtype must be uint8 or uint32')
# oldToQImage()

def convertToQImage(array, use_old=False):
    """Converts a numpy array to a QImage"""
    if use_old:
      image = oldToQImage(array)
    else:
      image = newToQImage(array)
    return image

class QwtPlotImage(QwtPlotItem):

    def __init__(self, parent, title =QwtText()):

        QwtPlotItem.__init__(self)
        self.plot = parent
        self.display_type = "hippo"
        self.ValueAxis =  None
        self.ComplexColorMap = None
        self._flags_array = None
        self._nan_flags_array = None
        self._image_for_display = None
        self._display_flags = False
        self.Qimage = None
        self.r_cmax = None
        self.r_cmin = None
        self.i_cmax = None
        self.i_cmin = None
        self.raw_image = None
        self.dimap = None
        self.complex = False
        self.log_scale = False
        self.log_y_scale = False
        self.transform_offset = 0.0
        self.flag_colour = 0
        self.nan_colour = 255
        self.lock_image_real = False
        self.lock_image_imag = False
        self.setTitle(title)
    # __init__()
    
    def setDisplayType(self, display_type):
      self.display_type = display_type
      if HAS_TIMBA:
        _dprint(2,'display type set to ', self.display_type);
    # setDisplayType

    def setFlagColour(self, flag_colour):
      self.flag_colour = flag_colour

    def setLockImage(self, real = True, lock_image=False):
      if real:
        self.lock_image_real = lock_image
      else:
        self.lock_image_imag = lock_image

    # When set to True, the log_scale parameter will cause
    # the displayed image to be scaled logarithmically.
    def setLogScale(self, log_scale = True):
      self.log_scale = log_scale
      if self.log_scale == False:
        self.dimap = None
        self.transform_offset = 0.0

    # When set to True, the log_y_scale parameter will cause
    # the output Y coordinate axis to be scaled properly for a Log plot
    # (used by QwtColorBar.py)
    def setLogYScale(self, log_y_scale = True):
      self.log_y_scale = log_y_scale

    def setFlagsArray(self, flags_array):
      self._flags_array = flags_array
      
    def setNanFlagsArray(self, flags_array):
      self._nan_flags_array = flags_array

    def setDisplayFlag(self, display_flags):
        self._display_flags = display_flags
    # setDisplayFlag

    def removeFlags(self):
      self._flags_array = None
      self._nan_flags_array = None
    # removeFlags

    def getRealImageRange(self):
      try:
        if self.raw_image.dtype == numpy.complex64 or self.raw_image.dtype == numpy.complex128:
          real_array =  self.raw_image.real
        else:
          real_array = self.raw_image
        return (self.r_cmin, self.r_cmax, real_array.min(),real_array.max())
      except:
        return (self.r_cmin, self.r_cmax, None, None)
    # getRealImageRange

    def getImagImageRange(self):
      try:
        if self.raw_image.dtype == numpy.complex64 or self.raw_image.dtype == numpy.complex128:
          imag_array =  self.raw_image.imag
        return (self.i_cmin, self.i_cmax,imag_array.min(), imag_array.max())
      except:
        return (self.i_cmin, self.i_cmax, None, None)
    # getRealImageRange


    
    def defineImageRange(self, limits, real=True):
       min = limits[0]
       max = limits[1]
       if abs(max - min) < 0.00005:
         if max == 0.0 or min == 0.0:
           min = -0.1
           max = 0.1
         else:
           min = 0.9 * min
           max = 1.1 * max
       if min > max:
         temp = max
         max = min
         min = temp
       if real:
         if not self.lock_image_real:
           self.r_cmin = min
           self.r_cmax = max
       else:
         if not self.lock_image_imag:
           self.i_cmin = min
           self.i_cmax = max

    def setImageRange(self, image):
      self.raw_image = image
      if image.dtype == numpy.complex64 or image.dtype == numpy.complex128:
        self.complex = True
        imag_array =  image.imag
        real_array =  image.real
        try:
          min = real_array.min()
          max = real_array.max()
        except:
          min = 0.0
          max = 0.0
        if abs(max - min) < 0.00005:
          if max == 0.0 or min == 0.0:
            min = -0.1
            max = 0.1
          else:
            min = 0.9 * min
            max = 1.1 * max
        if min > max:
          temp = max
          max = min
          min = temp
        if not self.lock_image_real:
          self.r_cmin = min
          self.r_cmax = max

        try:
          min = imag_array.min()
          max = imag_array.max()
        except:
          min = 0.0
          max = 0.0
        if abs(max - min) < 0.00005:
          if max == 0.0 or min == 0.0:
            min = -0.1
            max = 0.1
          else:
            min = 0.9 * min
            max = 1.1 * max
        if min > max:
          temp = max
          max = min
          min = temp
        if not self.lock_image_imag:
          self.i_cmin = min
          self.i_cmax = max
      else:
        self.complex = False
        try:
          min = image.min()
          max = image.max()
        except:
          min = 0.0
          max = 0.0
        if abs(max - min) < 0.00005:
          if max == 0.0 or min == 0.0:
            min = -0.1
            max = 0.1
          else:
            min = 0.9 * min
            max = 1.1 * max
        if min > max:
          temp = max
          max = min
          min = temp
        if not self.lock_image_real:
          self.r_cmin = min
          self.r_cmax = max
    # setImageRange

    def updateImage(self, image):
        self.setImage(image)
        self.raw_image = image

    def setFlaggedImageRange(self):
      (nx,ny) = self.raw_image.shape
      num_elements = nx * ny
      if self._flags_array is None:
        if not self._nan_flags_array is None:
          flags_array = self._nan_flags_array.copy()
        else:
          flags_array = numpy.zeros((nx,ny),int);
      else:
        flags_array = self._flags_array.copy()
        if not self._nan_flags_array is None:
          flags_array = flags_array + self._nan_flags_array
      flattened_flags = numpy.reshape(flags_array,(num_elements,))
      if self.raw_image.dtype == numpy.complex64 or self.raw_image.dtype == numpy.complex128:
        real_array =  self.raw_image.real
        imag_array =  self.raw_image.imag
        flattened_real_array = numpy.reshape(real_array.copy(),(num_elements,))
        flattened_imag_array = numpy.reshape(imag_array.copy(),(num_elements,))
        real_flagged_array = numpy.compress(flattened_flags == 0, flattened_real_array)
        imag_flagged_array = numpy.compress(flattened_flags == 0, flattened_imag_array)
        flagged_image = numpy.zeros(shape=real_flagged_array.shape,dtype=self.raw_image.dtype)
        flagged_image.real = real_flagged_array
        flagged_image.imag = imag_flagged_array
      else:
        flattened_array = numpy.reshape(self.raw_image.copy(),(num_elements,))
        flagged_image = numpy.compress(flattened_flags == 0, flattened_array)
      self.setImageRange(flagged_image)
    # setFlaggedImageRange

    def convert_to_log(self, incoming_image):
      self.transform_offset = 0.0
      transform_image = incoming_image
      image_min = incoming_image.min()
      if image_min <= 0.0:
        image_min = -1.0 * image_min
        self.transform_offset = 0.001 + image_min
        transform_image = self.transform_offset + incoming_image
      scale_min = transform_image.min()
      scale_max = transform_image.max()
      if scale_min == scale_max:
       scale_min = scale_min - 0.5 * scale_min
       scale_max = scale_max + 0.5 * scale_min
      self.dimap = ImageScaler(1, 256, scale_min, scale_max, True)
      if HAS_TIMBA:
        _dprint(3, 'doing log transform of ', transform_image)
      temp_image = self.dimap.iTransform(transform_image)
      if HAS_TIMBA:
        _dprint(3, 'log transformed image ', temp_image)
      return temp_image

    def getTransformOffset(self):
      return self.transform_offset

    def convert_limits(self, limits):
      if not self.dimap is None:
        first_limit = self.dimap.transform(self.transform_offset + limits[0])
        second_limit = self.dimap.transform(self.transform_offset + limits[1])
      else:
        first_limit = None
        second_limit = None
      return [first_limit, second_limit]


    def to_QImage(self, image):
# convert to 8 bit image
      image_for_display = None
      if image.dtype == numpy.complex64 or image.dtype == numpy.complex128:
        self.complex = True
        real_array =  image.real
        if self.log_scale:
          temp_array = self.convert_to_log(real_array)
          temp_array = self.convert_to_log(real_array)
          if not self.r_cmin is None and not self.r_cmax is None:
            limits = self.convert_limits([self.r_cmin,self.r_cmax])
          else:
            limits = [self.r_cmin, self.r_cmax] 
          byte_image = bytescale(temp_array,limits)
        else:
          limits = [self.r_cmin,self.r_cmax]
          byte_image = bytescale(real_array,limits)
        (nx,ny) = real_array.shape
        image_for_display = numpy.empty(shape=(nx*2,ny),dtype=byte_image.dtype);
        image_for_display[:nx,:] = byte_image
        imag_array =  image.imag
        if self.log_scale:
          temp_array = self.convert_to_log(imag_array)
          if not self.i_cmin is None and not self.i_cmax is None:
            limits = self.convert_limits([self.i_cmin,self.i_cmax])
          else:
            limits = [self.i_cmin, self.i_cmax] 
          byte_image = bytescale(temp_array,limits)
        else:
          limits = [self.i_cmin,self.i_cmax]
          byte_image = bytescale(imag_array,limits)
        image_for_display[nx:,:] = byte_image
      else:
        if self.log_scale:
          temp_array = self.convert_to_log(image)
          if not self.r_cmin is None and not self.r_cmax is None:
            limits = self.convert_limits([self.r_cmin,self.r_cmax])
          else:
            limits = [self.r_cmin, self.r_cmax] 
          #print 'to_QImage log limits = ', limits
          image_for_display = bytescale(temp_array,limits)
        else:
          limits = [self.r_cmin,self.r_cmax]
          #print 'to_QImage real limits = ', limits
          image_for_display = bytescale(image,limits)
      # turn image into a QImage, and return result   
      if not self._nan_flags_array is None:
        if self.complex:
          image_for_display[:nx,:] = numpy.where(self._nan_flags_array,1,image_for_display[:nx,:])
          image_for_display[nx:,:] = numpy.where(self._nan_flags_array,1,image_for_display[nx:,:])
        else:
          image_for_display = numpy.where(self._nan_flags_array,1,image_for_display)
      self._image_for_display = image_for_display
      result = convertToQImage(image_for_display,True).mirrored(0, 1)
      # always suppress NaNs
      if not self._nan_flags_array is None:
        result.setColor(1, qRgb(self.nan_colour, self.nan_colour, self.nan_colour))
      return result

    def toGrayScale(self, Qimage):
      for i in range(0, 256):
        Qimage.setColor(i, qRgb(i, i, i))

    def toHippo(self, Qimage):
      dv = 255.0
      vmin = 1.0
      for i in range(2, 256):
        r = 1.0
        g = 1.0
        b = 1.0
        v = 1.0 * i
        if (v < (vmin + 0.25 * dv)):
          r = 0;
          if dv != 0:
            g = 4 * (v - vmin) / dv;
        elif (v < (vmin + 0.5 * dv)):
          r = 0;
          if dv != 0:
            b = 1 + 4 * (vmin + 0.25 * dv - v) / dv;
        elif (v < (vmin + 0.75 * dv)):
          b = 0;
          if dv != 0:
            r = 4 * (v - vmin - 0.5 * dv) / dv;
        else: 
          b = 0;
          if dv != 0:
            g = 1 + 4 * (vmin + 0.75 * dv - v) / dv;
          else:
            r = 0
        red   = int ( r * 255. )
        green = int ( g * 255. )
        blue  = int ( b * 255. )
# the following call will use the previous computations to
# set up a hippo-like color display
        Qimage.setColor(i, qRgb(red, green, blue))

    def setImage(self, image):
# convert to QImage
      self.Qimage = self.to_QImage(image)
# set color scale a la HippoDraw Scale
      if self.display_type == "hippo":
        self.toHippo(self.Qimage)

# set color scale to Grayscale
      if self.display_type == "grayscale":
        self.toGrayScale(self.Qimage)

# compute flagged image if required
      if not self._flags_array is None or not self._nan_flags_array is None:
        self.setFlagQimage()

    def setFlagQimage(self):
      (nx,ny) = self._image_for_display.shape
      image_for_display = self._image_for_display.copy()
      if not self._flags_array is None:
        if self.complex:
          image_for_display[:nx//2,:] = numpy.where(self._flags_array,0,self._image_for_display[:nx//2,:])
          image_for_display[nx//2:,:] = numpy.where(self._flags_array,0,self._image_for_display[nx//2:,:])
        else:
          image_for_display = numpy.where(self._flags_array,0,self._image_for_display)

      if not self._nan_flags_array is None:
        if self.complex:
          image_for_display[:nx//2,:] = numpy.where(self._nan_flags_array,1,image_for_display[:nx//2,:])
          image_for_display[nx//2:,:] = numpy.where(self._nan_flags_array,1,image_for_display[nx//2:,:])
        else:
          image_for_display = numpy.where(self._nan_flags_array,1,image_for_display)

      self.flags_Qimage = convertToQImage(image_for_display,True).mirrored(0, 1)

# set color scale a la HippoDraw Scale
      if self.display_type == "hippo":
        self.toHippo(self.flags_Qimage)

# set color scale to Grayscale
      if self.display_type == "grayscale":
        self.toGrayScale(self.flags_Qimage)

# set zero to black to display flag image pixels in black 
      self.flags_Qimage.setColor(0, qRgb(self.flag_colour, self.flag_colour, self.flag_colour))
      self.flags_Qimage.setColor(1, qRgb(self.nan_colour, self.nan_colour, self.nan_colour))

    def setBrentjensImage(self, image):
      absmin = abs(image.min())
      MaxAbs = abs(image.max())
      if (absmin > MaxAbs):
        MaxAbs = absmin
      self.ValueAxis.calcTransferFunction(-MaxAbs, MaxAbs, 0, self.ComplexColorMap.getNumberOfColors()-1)

      if image.min() != image.max():
# get real and imaginary arrays
        real_image = image.real
        imag_image = image.imag
        shape = image.shape
        Ncol = self.ComplexColorMap.getNumberOfColors()
        bits_per_pixel = 32
        self.Qimage = QImage(shape[0], shape[1], bits_per_pixel, Ncol)
        for i in range(shape[0]):
          for j in range(shape[1]):
            colre = int(self.ValueAxis.worldToAxis(real_image[i,j]))
            colim = int(self.ValueAxis.worldToAxis(imag_image[i,j]))
            if(colre < Ncol and colim < Ncol): 
              value = self.ComplexColorMap.get_color_value(colre,colim)
              self.Qimage.setPixel(i,j,value)
            else:
              if HAS_TIMBA:
               _dprint(2, "*************************************");
               _dprint(2, "colre: ", colre);
               _dprint(2, "colim: ", colim);
               _dprint(2, "real : ", real_image[i,j]);
               _dprint(2, "imag : ", imag_image[i,j]);
               _dprint(2, "Ncol: ", Ncol);
               _dprint(2, "*************************************");
        self.Qimage.mirror(0,1)

    def setData(self, data_array, xScale = None, yScale = None):
        '*** setting data **** '
        self.complex = False
        shape = data_array.shape
        if HAS_TIMBA:
          _dprint(3, 'array shape is ', shape)
        shape0 = shape[0]
        if data_array.dtype == numpy.complex64 or data_array.dtype == numpy.complex128:
          self.complex = True
          shape0 = 2 * shape[0]
        if xScale:
            self.xMap = ImageScaler(0, shape0 - 1, xScale[0], xScale[1])
            self.xMap_draw = ImageScaler(0, shape0 - 1, xScale[0], xScale[1])
            temp_scale = (xScale[0],xScale[1])
            self.plot.setAxisScale(QwtPlot.xBottom, *temp_scale)
            if HAS_TIMBA:
             _dprint(3, 'xScale is ', xScale)
        else:
            self.xMap = ImageScaler(0, shape0, 0, shape0 )
            self.xMap_draw = ImageScaler(0, shape0, 0, shape0 )
            self.plot.setAxisScale(QwtPlot.xBottom, 0, shape0)
        if yScale:
            if HAS_TIMBA:
             _dprint(3, 'yScale is ', yScale)
             _dprint(3, 'self.log_y_scale is ', self.log_y_scale)

            self.yMap = ImageScaler(0, shape[1]-1, yScale[0], yScale[1],self.log_y_scale)
            self.yMap_draw = ImageScaler(0, shape[1]-1, yScale[0], yScale[1],self.log_y_scale)
            temp_scale = (yScale[0],yScale[1])
            if HAS_TIMBA:
             _dprint(3, 'Called setAxisScale(QwtPlot.yLeft) with ', temp_scale)
            self.plot.setAxisScale(QwtPlot.yLeft, *temp_scale)
        else:
            self.yMap = ImageScaler(0, shape[1], 0, shape[1])
            self.yMap_draw = ImageScaler(0, shape[1], 0, shape[1])
            self.plot.setAxisScale(QwtPlot.yLeft, 0, shape[1])
        self.setImage(data_array)
        self.raw_image = data_array
    # setData()    

    def update_yMap_draw(self, d1, d2):
      self.yMap_draw.setDblRange(d1, d2, self.log_y_scale)

    def update_xMap_draw(self, d1, d2):
      self.xMap_draw.setDblRange(d1, d2, self.log_y_scale)

    def get_xMap_draw_coords(self):
      return (self.xMap_draw.d1(), self.xMap_draw.d2())

    def get_yMap_draw_coords(self):
      return (self.yMap_draw.d1(), self.yMap_draw.d2())

    def draw(self, painter, xMap, yMap,rect):
        """Paint image to zooming to xMap, yMap

        Calculate (x1, y1, x2, y2) so that it contains at least 1 pixel,
        and copy the visible region to scale it to the canvas.

        NOTE: we ignore the system-provided Maps and use our own 
        'draw' maps
        """
        if self.Qimage is None:
          return

#       #_dprint(3, 'incoming x map ranges ',xMap.s1(), ' ', xMap.s2())
#       #_dprint(3, 'incoming y map ranges ',yMap.s1(), ' ', yMap.s2())
#       print 'incoming x map ranges ',xMap.s1(), ' ', xMap.s2()
#       print 'incoming y map ranges ',yMap.s1(), ' ', yMap.s2()
#       print 'incoming x map draw ranges ',self.xMap_draw.d1(), ' ', self.xMap_draw.d2()
#       print 'incoming y map draw ranges ',self.yMap_draw.d1(), ' ', self.yMap_draw.d2()
#       print 'incoming self x map ranges ',self.xMap.d1(), ' ', self.xMap.d2()
#       print 'incoming self y map ranges ',self.yMap.d1(), ' ', self.yMap.d2()

        # calculate y1, y2
        y1 = y2 = self.Qimage.height()
        if HAS_TIMBA:
         _dprint(3, 'image height ', self.Qimage.height())
#        y1 = y2 = self.Qimage.height() - 1
#       print 'starting image height ', y1
        y1 *= (self.yMap.d2() - self.yMap_draw.d2())
        y1 /= (self.yMap.d2() - self.yMap.d1())
#       y1 = max(0, int(y1-0.5))
        y1 = max(0, (y1-0.5))
        if HAS_TIMBA:
          _dprint(3, 'float y1 ', y1)
        y1 = int(y1 + 0.5)
        y2 *= (self.yMap.d2() - self.yMap_draw.d1())
        y2 /= (self.yMap.d2() - self.yMap.d1())
        if HAS_TIMBA:
         _dprint(3, 'float y2 ', y2)
#       y2 = min(self.Qimage.height(), int(y2+0.5))
        y2 = min(self.Qimage.height(), (y2+0.5))
        y2 = int(y2)
        if HAS_TIMBA:
         _dprint(3, 'int y1, y2 ', y1, ' ',y2)
        # calculate x1, x2 - these are OK
        x1 = x2 = self.Qimage.width() 
#       print 'starting image width ', x1
        x1 *= (xMap.s1() - self.xMap.d1())
        x1 /= (self.xMap.d2() - self.xMap.d1())
        if HAS_TIMBA:
         _dprint(3, 'float x1 ', x1)
#       x1 = max(0, int(x1-0.5))
        x1 = max(0, int(x1))
        x2 *= (self.xMap_draw.d2() - self.xMap.d1())
        x2 /= (self.xMap.d2() - self.xMap.d1())
        if HAS_TIMBA:
         _dprint(3, 'float x2 ', x2)
        x2 = min(self.Qimage.width(), int(x2+0.5))
        if HAS_TIMBA:
         _dprint(3, 'int x1, x2 ', x1, ' ',x2)
        # copy
        xdelta = x2-x1
        ydelta = y2-y1
#       print 'xdelta ydelta ', xdelta, ' ', ydelta
#       print 'x1 y1 ', x1, ' ', y1
        # these tests seem necessary for the dummy 'scalar' displays
        if xdelta < 0:
          xdelta = self.Qimage.height()
        if ydelta < 0:
          ydelta = self.Qimage.height()
        image = None
        if self._display_flags:
          image = self.flags_Qimage.copy(x1, y1, xdelta, ydelta)
        else:
          image = self.Qimage.copy(x1, y1, xdelta, ydelta)
        # zoom
        image = image.scaled(xMap.p2()-xMap.p1()+1, yMap.p1()-yMap.p2()+1)
        # draw
        painter.drawImage(xMap.p1(), yMap.p2(), image)

    # drawImage()

# QwtPlotImage()

# we test the QwtPlotImage class with class QwtImagePlot

class QwtImagePlot(QwtPlot):
    def __init__(self, *args):
        QwtPlot.__init__(self, *args)
	# make a QwtPlot widget
        self.plotLayout().setCanvasMargin(0)
        self.plotLayout().setAlignCanvasToScales(1)
        self.setTitle('QwtImagePlot: (un)zoom & (un)hide')
	# set axis titles
        self.setAxisTitle(QwtPlot.xBottom, 'time (s)')
        self.setAxisTitle(QwtPlot.yLeft, 'frequency (Hz)')
	# insert a few curves
        self.cSin = QwtPlotCurve('y = pi*sin(x)')
        self.cCos = QwtPlotCurve('y = 4*pi*sin(x)*cos(x)**2')
        self.cSin.attach(self)
        self.cCos.attach(self)
	# set curve styles
        self.cSin.setPen(QPen(Qt.green, 2))
        self.cCos.setPen(QPen(Qt.black, 2))
        self.xzoom_loc = None
        self.yzoom_loc = None
        self.xpos = None
        self.ypos = None

	# attach a grid
        grid = QwtPlotGrid()
        grid.attach(self)
        grid.setPen(Qt.black, 0, Qt.DotLine)

        # create zoom curve
        self.zoom_outline = QwtPlotCurve()
        self.zoom_outline.setStyle(QwtPlotCurve.Lines) 

        # create and initialize an image display
        self.plotImage = QwtPlotImage(self)
        self.plotImage.attach(self)
        self.gain = 2.0
        self.updateDisplay()

        self.zoomStack = []

        self.spy = Spy(self.canvas())
        self.prev_xpos = None
        self.prev_ypos = None

#       self.connect(self, Qt.SIGNAL("legendClicked(QwtPlotItem*)"),
#                    self.toggleVisibility)

        self.spy.MouseMove.connect(self.onmouseMoveEvent)
        self.spy.MousePress.connect(self.onmousePressEvent)
        self.spy.MouseRelease.connect(self.onmouseReleaseEvent)


    # __init__()

    def toggleVisibility(self, plotItem):
        """Toggle the visibility of a plot item
        """
        plotItem.setVisible(not plotItem.isVisible())
        self.replot()



    # showCoordinates()


    def updateDisplay(self):
      offset = 3
      # calculate 3 NumPy arrays
      x = numpy.arange(-1.0 * self.gain*math.pi, self.gain*math.pi, 0.01)
      y = math.pi*numpy.sin(x) + offset
      z = self.gain * self.gain*math.pi*numpy.cos(x)*numpy.cos(x)*numpy.sin(x) + offset
      # copy the data
      self.cSin.setSamples(x, y)
      self.cCos.setSamples(x, z)
      # image
      test_image = sinx_image(8,16,offset)
      self.plotImage.setData(test_image)

#     self.plotImage.setData(
#           square(512,-1.0 * self.gain*math.pi, self.gain*math.pi), (-1.0*self.gain*math.pi, self.gain*math.pi), (-1.0*self.gain*math.pi, self.gain*math.pi))

    def updateBarDisplay(self):
      self.min = 0.0
      self.max = 256.0
      self.bar_array = numpy.reshape(numpy.arange(self.max), (1,256))
      self.y_scale = (self.min, self.max)
      self.plotImage.setData(self.bar_array, None, self.y_scale)

    def start_timer(self, time):
      """ start a timer going to update the image every 1/10 sec """
      self.timer = QTimer(self)
      self.timer.timeout.connect(self.testEvent)
      self.timer.start(time)

    def testEvent(self):
      """ change the gain factor and recalculate the image """
      self.gain = self.gain + 1.0
      self.updateDisplay()
      self.updateBarDisplay()
      self.replot()


    def mouseCatch(self,x,y,z):
      print('in mouseCatch')
      print('caught', x,y, z)

    def onmouseMoveEvent(self,event):
      position = event.pos()
      x = position.x()
      y = position.y()
      button = event.button()
      if not self.xpos is None:
        self.xpos = self.invTransform(QwtPlot.xBottom, x)
        self.ypos = self.invTransform(QwtPlot.yLeft, y)
        if not self.xzoom_loc is None:
          self.xzoom_loc = [self.press_xpos, self.press_xpos,  self.xpos, self.xpos,self.press_xpos]
          self.yzoom_loc = [self.press_ypos, self.ypos,  self.ypos, self.press_ypos,self.press_ypos]
          self.zoom_outline.setSamples(self.xzoom_loc,self.yzoom_loc)
          self.replot()

    def onmousePressEvent(self,event):
        position = event.pos()
        x = position.x()
        y = position.y()
        button = event.button()
        if Qt.LeftButton == button:
            if self.xpos is None:
              self.xpos = self.invTransform(QwtPlot.xBottom, x)
              self.ypos = self.invTransform(QwtPlot.yLeft, y)
            # Python semantics: self.pos = e.pos() does not work; force a copy
            self.press_xpos = self.xpos
            self.press_ypos = self.ypos
            self.xzoom_loc = [self.press_xpos]
            self.yzoom_loc = [self.press_ypos]
            self.zoom_outline.attach(self)
            self.zooming = 1
            if self.zoomStack == []:
                try:
                  self.zoomState = (
                    self.axisScaleDiv(QwtPlot.xBottom).lBound(),
                    self.axisScaleDiv(QwtPlot.xBottom).hBound(),
                    self.axisScaleDiv(QwtPlot.yLeft).lBound(),
                    self.axisScaleDiv(QwtPlot.yLeft).hBound(),
                    )
                except:
                  self.zoomState = (
                    self.axisScaleDiv(QwtPlot.xBottom).lowerBound(),
                    self.axisScaleDiv(QwtPlot.xBottom).upperBound(),
                    self.axisScaleDiv(QwtPlot.yLeft).lowerBound(),
                    self.axisScaleDiv(QwtPlot.yLeft).upperBound(),
                    )
        elif Qt.RightButton == button:
            self.zooming = 0
    # mousePressEvent()

    def onmouseReleaseEvent(self,event):
        position = event.pos()
        x = position.x()
        y = position.y()
        button = event.button()
        if Qt.LeftButton == button:
            xmin = min(self.xpos, self.press_xpos)
            xmax = max(self.xpos, self.press_xpos)
            ymin = min(self.ypos, self.press_ypos)
            ymax = max(self.ypos, self.press_ypos)
            if not self.xzoom_loc is None:
              self.zoom_outline.detach()
              self.xzoom_loc = None
              self.yzoom_loc = None
            if xmin == xmax or ymin == ymax:
                return
            self.zoomStack.append(self.zoomState)
            self.zoomState = (xmin, xmax, ymin, ymax)
        elif Qt.RightButton == button:
            # we back up the zoom stack
            if len(self.zoomStack):
                xmin, xmax, ymin, ymax = self.zoomStack.pop()
            else:
                return

        if button != Qt.MidButton:
          self.setAxisScale(QwtPlot.xBottom, xmin, xmax)
          self.setAxisScale(QwtPlot.yLeft, ymin, ymax)
          self.plotImage.update_xMap_draw(xmin,xmax)
          self.plotImage.update_yMap_draw(ymin,ymax)
          self.replot()

    # mouseReleaseEvent()


    def toggleCurve(self, key):
        curve = self.curve(key)
        if curve:
            curve.setEnabled(not curve.enabled())
            self.replot()

    # toggleCurve()

# class QwtImagePlot


def make():
    demo = QwtImagePlot()
    demo.resize(500, 300)
#   demo.start_timer(500)
    demo.show()
    return demo

# make()



def main(args):
    app = QApplication(sys.argv)
    demo = make()
#   app.setMainWidget(demo)
    app.exec_()

# main(

# Admir
if __name__ == '__main__':
    import sys
    main(sys.argv)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
# Copyright (C) 2002-2007
# ASTRON (Netherlands Foundation for Research in Astronomy)
# and The MeqTree Foundation
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#  (c) 2013.				 (c) 2011.
#  National Research Council		 Conseil national de recherches
#  Ottawa, Canada, K1A 0R6 		 Ottawa, Canada, K1A 0R6
#
#  This software is free software;	 Ce logiciel est libre, vous
#  you can redistribute it and/or	 pouvez le redistribuer et/ou le
#  modify it under the terms of	         modifier selon les termes de la
#  the GNU General Public License	 Licence Publique Generale GNU
#  as published by the Free		 publiee par la Free Software
#  Software Foundation; either	 	 Foundation (version 3 ou bien
#  version 2 of the License, or	 	 toute autre version ulterieure
#  (at your option) any later	 	 choisie par vous).
#  version.
#
#  This software is distributed in	 Ce logiciel est distribue car
#  the hope that it will be		 potentiellement utile, mais
#  useful, but WITHOUT ANY		 SANS AUCUNE GARANTIE, ni
#  WARRANTY; without even the	 	 explicite ni implicite, y
#  implied warranty of			 compris les garanties de
#  MERCHANTABILITY or FITNESS FOR	 commercialisation ou
#  A PARTICULAR PURPOSE.  See the	 d'adaptation dans un but
#  GNU General Public License for	 specifique. Reportez-vous a la
#  more details.			 Licence Publique Generale GNU
#  					 pour plus de details.
#
#  You should have received a copy	 Vous devez avoir recu une copie
#  of the GNU General Public		 de la Licence Publique Generale
#  License along with this		 GNU en meme temps que ce
#  software; if not, contact the	 logiciel ; si ce n'est pas le
#  Free Software Foundation, Inc.	 cas, communiquez avec la Free
#  at http://www.fsf.org.		 Software Foundation, Inc. au
#						 http://www.fsf.org.
#
#  email:				 courriel:
#  business@hia-iha.nrc-cnrc.gc.ca	 business@hia-iha.nrc-cnrc.gc.ca
#
#  National Research Council		 Conseil national de recherches
#      Canada				    Canada
#  Herzberg Institute of Astrophysics	 Institut Herzberg d'astrophysique
#  5071 West Saanich Rd.		 5071 West Saanich Rd.
#  Victoria BC V9E 2E7			 Victoria BC V9E 2E7
#  CANADA					 CANADA
#

# this is a python translation of the ACSIS c++ zoomwin.cc program
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

import sys
import numpy
import numpy

from qwt.qt.QtGui import QApplication, QHBoxLayout, QWidget
from qwt.qt.QtCore import Qt, pyqtSignal
from qwt import QwtPlot, QwtPlotCurve

from MeqGUI.Plugins.display_image_qt5 import QwtImageDisplay

#widget to show a zoomed chanel of the plot

class ZoomPopup(QWidget):

  winclosed = pyqtSignal(int)
  winpaused = pyqtSignal(int)
  save_zoom_display = pyqtSignal(str,int)
  image_auto_scale = pyqtSignal(int)
  image_scale_values = pyqtSignal(float, float)
 

  def __init__(self, CurveNumber, x_values, y_values , flags, pen, parent=None, name=None):
    """ Initialises all the variables.  
        creates the main zoom plot
        connects the qt signals
    """


    QWidget.__init__(self, parent)
    self.setWindowTitle('Channel ' + str(CurveNumber))
    self._parent = parent
    self._d_zoomActive = self._d_zoom = False
    self._curve_number = CurveNumber
    self.curves = {}

    self._do_close = True   # enable closing by window manager
    self._do_pause = False   # pause mode is False at startup
    self._compare_max = False
    self._do_linear_scale = True  # linear Y axis scale by default
    self._do_fixed_scale = False  # auto scaling by default
    self._array_label = "Channel "
 
    #Create the plot for selected curve to zoom
    self._plotter = QwtImageDisplay(self)
    self._plotter.setZoomDisplay()

    self._zoom_plot_label = self._array_label + str(self._curve_number) + " Sequence (oldest to most recent)"

    self._max_crv = -1  # negative value used to indicate that this display
    self._min_crv = -1  # is not being used

  #####end of parameters set for the plot#######/

    # we seem to need a layout for PyQt
    box1 = QHBoxLayout( self )
    box1.addWidget(self._plotter)
#   self.plotPrinter = plot_printer_qt5.plot_printer(self._plotter)

    self._plotter.winpaused.connect(self.Pausing)
    self._plotter.compare.connect(self.do_compare)
#   self._plotter.do_print.connnect(self.plotPrinter.do_print)
    self._plotter.save_display.connect(self.handle_save_display)


    # insert flags ?   
    self._plotter.initVellsContextMenu()
    self.update_plot(y_values, flags)
    self.show()

  def handle_save_display(self, title):
    self.save_zoom_display.emit(self._zoom_plot_label, self._curve_number)

  def do_compare_max(self, x_values):
    ### instantiate the envelop that will show min/max deviations
    self._max_envelop = self._y_values
    self._min_envelop = self._y_values
    self._max_crv = QwtPlotCurve('Zoomed max curve')
    self._max_crv.attach(self._plotter)
    self._min_crv = QwtPlotCurve('Zoomed min curve')
    self._min_crv.attach(self._plotter)
    self._max_crv.setData(x_values,self._max_envelop)
    self._min_crv.setData(x_values,self._min_envelop)
    self._compare_max = True

  def do_compare(self):
    print('in zoomwin do_compare')
    if self._compare_max:
      self.stop_compare_max()
      self._compare_max = False
    else:
      self._max_envelop = self._y_values
      self._min_envelop = self._y_values
      self._max_crv = QwtPlotCurve('Zoomed max curve')
      self._max_crv.attach(self._plotter)
      self._min_crv = QwtPlotCurve('Zoomed min curve')
      self._min_crv.attach(self._plotter)
      self._max_crv.setData(x_values,self._max_envelop)
      self._min_crv.setData(x_values,self._min_envelop)
      self._compare_max = True
    self.reset_max()

  def stop_compare_max(self):
    if self._compare_max:
      self._max_envelop = 0.0
      self._min_envelop = 0.0
      self._max_crv.detach()
      self._min_crv.detach()
      self._compare_max = False
      self._max_crv = -1
      self._min_crv = -1

  def get_max(self):
    if self._compare_max:
      self._max_envelop = self.max(self._max_envelop, self._y_values)
      self._min_envelop = self.min(self._min_envelop, self._y_values)

  def max(self, array1, array2):
     shape = array1.shape
     max_envelop = array1
     for i in range(shape[0]):
       if array2[i] > array1[i]:
         max_envelop[i] = array2[i]
     return max_envelop

  def min(self, array1, array2):
     shape = array1.shape
     min_envelop = array1
     for i in range(shape[0]):
       if array2[i] < array1[i]:
         min_envelop[i] = array2[i]
     return min_envelop

  def reset_max(self):
    if self._compare_max:
      self._max_envelop = self._y_values
      self._min_envelop = self._y_values

  def test_max(self):
    if self._compare_max: 
      return True
    else:
      return False

  def pause_mode(self):
    if self._do_pause: 
      return True
    else:
      return False

  def exec_close(self):
    self.close()

  def Pausing(self):
    self.winpaused.emit(self._curve_number)

  def change_scale_type(self):
# click means change to fixed scale
    toggle_id = self.menu_table['Fixed Scale ']
    if self._do_fixed_scale:
      self._do_fixed_scale = False
      self._menu.changeItem(toggle_id, 'Fixed Scale')
      self._plotter.setAxisAutoScale(QwtPlot.yLeft)
      self.image_auto_scale.emit(0)
    else:
      self._do_fixed_scale = True
      self._menu.changeItem(toggle_id, 'Auto Scale')
# find current data min and max
      scale_max = self._y_values.max()
      scale_min = self._y_values.min()

  def set_scale_values(self,max_value,min_value):
    if self._do_fixed_scale:
      self.image_scale_values.emit(max_value,min_value)
      self._plotter.setAxisScale(QwtPlot.yLeft, min_value, max_value)
      self._plotter.replot()

  def cancel_scale_request(self):
    if self._do_fixed_scale:
      toggle_id = self.menu_table['Fixed Scale ']
      self._menu.changeItem(toggle_id, 'Fixed Scale')
      self._plotter.setAxisAutoScale(QwtPlot.yLeft)
      self._do_fixed_scale = False
  
  def update_plot(self,y_values, flags):
    if not self._do_pause:
      self._plotter.unsetFlagsData()
      self._y_values = y_values
      abs_flags = numpy.absolute(flags)
      if abs_flags.max() > 0:
        if len(flags) == len(self._y_values):
          self._plotter.setFlagsData(flags,flip_axes=True)
          self._plotter.set_flag_toggles_active(True,False)
      else:
        self._plotter.set_flag_toggles_active(False,False)
      self._plotter.array_plot(incoming_plot_array=self._y_values, flip_axes=True)

#     self.get_max()
      self._plotter.replot()

  def setDataLabel(self, data_label, array_label, is_array=False):
    self._data_label = data_label
    if array_label is None:
      self._array_label = 'Ch ' + str(self._curve_number)
    else:
      self._array_label = array_label
    if is_array:
      self._zoom_plot_label = self._data_label + ": " + self._array_label  
    else:
      self._zoom_plot_label = self._data_label + ": " + self._array_label + " Sequence (oldest to most recent)"
    self._plotter.setAxisTitle(QwtPlot.xBottom, self._zoom_plot_label)
    self._plotter._x_title = self._zoom_plot_label;
    self.setWindowTitle(self._zoom_plot_label)

  def plotMouseMoved(self, e):
    """	Gets x and y position of the mouse on the plot according to axis' value
        set right text on the button and underneath the plot
    """
# (I) e (QMouseEvent) Mouse event
    lbl = QString("Event=")
    lbl2 = QString("")
    lbl2.setNum(self._plotter.invTransform(QwtPlot.xBottom, e.pos().x() ), 'g', 3)
    lbl += lbl2 + ",  Signal="
    lbl2.setNum(self._plotter.invTransform(QwtPlot.yLeft, e.pos().y() ), 'g', 3)
    lbl += lbl2
#   self._ControlFrame._lblInfo.setText(lbl)

  def closeEvent(self, ce):
    if self._do_close:
      self.winclosed.emit(self._curve_number)
      ce.accept()
    else:
      ce.ignore()
    #---------------------------------------------
    # Don't call the base function because
    # we want to ignore the close event
    #---------------------------------------------

def main(args):
    app = QApplication(args)
    x_values = numpy.array([1.0,2.0,3.0,4.0])
    y_values = numpy.array([15.0,25.0,35.0,45.0])
    flags = numpy.array([0.0,0.0,0.0,0.0])
    demo = ZoomPopup(1, x_values, y_values,flags, Qt.yellow,None, None )
    demo.show()
    app.exec_()


# Admire
if __name__ == '__main__':
    main(sys.argv)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dialog_window_config.ui'
#
# Created: Tue Aug 30 11:19:40 2005
#      by: The PyQt User Interface Compiler (pyuic) 3.13
#
# WARNING! All changes made in this file will be lost!

# A script to put up a GUI so that the user can modify plot settings
# Gratefully adapted by AGW from the SciCraft 'dialog_window_config.ui' 


#% $Id$ 

#
# Copyright (C) 2002-2007
# ASTRON (Netherlands Foundation for Research in Astronomy)
# and The MeqTree Foundation
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

from qt import *
import sys

class dialog_window_config(QDialog):
    def __init__(self,parent = None,name = None, mother=None, modal = 0,fl = 0):
        QDialog.__init__(self,parent,name,modal,fl)


# The following two lines seem wierd, but are necessary until I figure
# out why they are necessary! (or not!)
        self.mother = mother
        if not name:
            self.setName("dialog_window_config")

        self.setSizeGripEnabled(1)
        self.setModal(0)

        dialog_window_configLayout = QVBoxLayout(self,11,6,"dialog_window_configLayout")

        self.tab_widget_categories = QTabWidget(self,"tab_widget_categories")

        self.Widget2 = QWidget(self.tab_widget_categories,"Widget2")
        Widget2Layout = QGridLayout(self.Widget2,1,1,11,6,"Widget2Layout")

        self.group_box_window_labels = QGroupBox(self.Widget2,"group_box_window_labels")
        self.group_box_window_labels.setColumnLayout(0,Qt.Vertical)
        self.group_box_window_labels.layout().setSpacing(6)
        self.group_box_window_labels.layout().setMargin(11)
        group_box_window_labelsLayout = QGridLayout(self.group_box_window_labels.layout())
        group_box_window_labelsLayout.setAlignment(Qt.AlignTop)

        self.line_edit_x_axis_label = QLineEdit(self.group_box_window_labels,"line_edit_x_axis_label")

        group_box_window_labelsLayout.addWidget(self.line_edit_x_axis_label,1,1)

        self.text_label_y_axis_label = QLabel(self.group_box_window_labels,"text_label_y_axis_label")

        group_box_window_labelsLayout.addWidget(self.text_label_y_axis_label,2,0)

        self.text_label_window_title = QLabel(self.group_box_window_labels,"text_label_window_title")

        group_box_window_labelsLayout.addWidget(self.text_label_window_title,0,0)

        self.line_edit_window_title = QLineEdit(self.group_box_window_labels,"line_edit_window_title")

        group_box_window_labelsLayout.addWidget(self.line_edit_window_title,0,1)

        self.text_label_x_axis_label = QLabel(self.group_box_window_labels,"text_label_x_axis_label")

        group_box_window_labelsLayout.addWidget(self.text_label_x_axis_label,1,0)

        self.line_edit_y_axis_label = QLineEdit(self.group_box_window_labels,"line_edit_y_axis_label")

        group_box_window_labelsLayout.addWidget(self.line_edit_y_axis_label,2,1)

        Widget2Layout.addWidget(self.group_box_window_labels,0,0)
        self.tab_widget_categories.insertTab(self.Widget2,QString(""))

        self.TabPage = QWidget(self.tab_widget_categories,"TabPage")
        TabPageLayout = QHBoxLayout(self.TabPage,11,6,"TabPageLayout")

        self.group_box_major_grid = QGroupBox(self.TabPage,"group_box_major_grid")
        self.group_box_major_grid.setColumnLayout(0,Qt.Vertical)
        self.group_box_major_grid.layout().setSpacing(6)
        self.group_box_major_grid.layout().setMargin(11)
        group_box_major_gridLayout = QGridLayout(self.group_box_major_grid.layout())
        group_box_major_gridLayout.setAlignment(Qt.AlignTop)

        self.text_label_maj_grid_width = QLabel(self.group_box_major_grid,"text_label_maj_grid_width")
        self.text_label_maj_grid_width.setEnabled(0)

        group_box_major_gridLayout.addWidget(self.text_label_maj_grid_width,3,0)

        self.text_label_maj_grid_style = QLabel(self.group_box_major_grid,"text_label_maj_grid_style")
        self.text_label_maj_grid_style.setEnabled(0)

        group_box_major_gridLayout.addWidget(self.text_label_maj_grid_style,4,0)

        self.text_label_maj_grid_colour = QLabel(self.group_box_major_grid,"text_label_maj_grid_colour")
        self.text_label_maj_grid_colour.setEnabled(0)

        group_box_major_gridLayout.addWidget(self.text_label_maj_grid_colour,2,0)

        self.check_box_x_grid_maj_enable = QCheckBox(self.group_box_major_grid,"check_box_x_grid_maj_enable")
        self.check_box_x_grid_maj_enable.setChecked(0)

        group_box_major_gridLayout.addMultiCellWidget(self.check_box_x_grid_maj_enable,0,0,0,1)

        self.check_box_y_grid_maj_enable = QCheckBox(self.group_box_major_grid,"check_box_y_grid_maj_enable")
        self.check_box_y_grid_maj_enable.setChecked(0)

        group_box_major_gridLayout.addMultiCellWidget(self.check_box_y_grid_maj_enable,1,1,0,1)

        layout_maj_grid_colour = QHBoxLayout(None,0,6,"layout_maj_grid_colour")
        spacer_maj_grid_colour_left = QSpacerItem(27,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout_maj_grid_colour.addItem(spacer_maj_grid_colour_left)

        self.push_button_maj_grid_colour = QPushButton(self.group_box_major_grid,"push_button_maj_grid_colour")
        self.push_button_maj_grid_colour.setEnabled(0)
        self.push_button_maj_grid_colour.setPaletteBackgroundColor(QColor(0,0,0))
        layout_maj_grid_colour.addWidget(self.push_button_maj_grid_colour)
        spacer_maj_grid_colour_right = QSpacerItem(28,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout_maj_grid_colour.addItem(spacer_maj_grid_colour_right)

        group_box_major_gridLayout.addLayout(layout_maj_grid_colour,2,1)

        self.spin_box_maj_grid_width = QSpinBox(self.group_box_major_grid,"spin_box_maj_grid_width")
        self.spin_box_maj_grid_width.setEnabled(0)

        group_box_major_gridLayout.addWidget(self.spin_box_maj_grid_width,3,1)

        self.combo_box_maj_grid_style = QComboBox(0,self.group_box_major_grid,"combo_box_maj_grid_style")
        self.combo_box_maj_grid_style.setEnabled(0)

        group_box_major_gridLayout.addWidget(self.combo_box_maj_grid_style,4,1)
        TabPageLayout.addWidget(self.group_box_major_grid)

        self.group_box_minor_grid = QGroupBox(self.TabPage,"group_box_minor_grid")
        self.group_box_minor_grid.setColumnLayout(0,Qt.Vertical)
        self.group_box_minor_grid.layout().setSpacing(6)
        self.group_box_minor_grid.layout().setMargin(11)
        group_box_minor_gridLayout = QGridLayout(self.group_box_minor_grid.layout())
        group_box_minor_gridLayout.setAlignment(Qt.AlignTop)

        self.text_label_min_grid_colour = QLabel(self.group_box_minor_grid,"text_label_min_grid_colour")
        self.text_label_min_grid_colour.setEnabled(0)

        group_box_minor_gridLayout.addWidget(self.text_label_min_grid_colour,2,0)

        self.text_label_min_grid_width = QLabel(self.group_box_minor_grid,"text_label_min_grid_width")
        self.text_label_min_grid_width.setEnabled(0)

        group_box_minor_gridLayout.addWidget(self.text_label_min_grid_width,3,0)

        self.text_label_min_grid_style = QLabel(self.group_box_minor_grid,"text_label_min_grid_style")
        self.text_label_min_grid_style.setEnabled(0)
        self.text_label_min_grid_style.setFocusPolicy(QLabel.NoFocus)

        group_box_minor_gridLayout.addWidget(self.text_label_min_grid_style,4,0)

        self.check_box_y_grid_min_enable = QCheckBox(self.group_box_minor_grid,"check_box_y_grid_min_enable")
        self.check_box_y_grid_min_enable.setEnabled(0)

        group_box_minor_gridLayout.addMultiCellWidget(self.check_box_y_grid_min_enable,1,1,0,1)

        self.check_box_x_grid_min_enable = QCheckBox(self.group_box_minor_grid,"check_box_x_grid_min_enable")
        self.check_box_x_grid_min_enable.setEnabled(0)
        self.check_box_x_grid_min_enable.setChecked(0)

        group_box_minor_gridLayout.addMultiCellWidget(self.check_box_x_grid_min_enable,0,0,0,1)

        layout_min_grid_colour = QHBoxLayout(None,0,6,"layout_min_grid_colour")
        spacer_min_grid_colour_left = QSpacerItem(27,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout_min_grid_colour.addItem(spacer_min_grid_colour_left)

        self.push_button_min_grid_colour = QPushButton(self.group_box_minor_grid,"push_button_min_grid_colour")
        self.push_button_min_grid_colour.setEnabled(0)
        self.push_button_min_grid_colour.setPaletteBackgroundColor(QColor(0,0,0))
        layout_min_grid_colour.addWidget(self.push_button_min_grid_colour)
        spacer_min_grid_colour_right = QSpacerItem(28,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        layout_min_grid_colour.addItem(spacer_min_grid_colour_right)

        group_box_minor_gridLayout.addLayout(layout_min_grid_colour,2,1)

        self.spin_box_min_grid_width = QSpinBox(self.group_box_minor_grid,"spin_box_min_grid_width")
        self.spin_box_min_grid_width.setEnabled(0)

        group_box_minor_gridLayout.addWidget(self.spin_box_min_grid_width,3,1)

        self.combo_box_min_grid_style = QComboBox(0,self.group_box_minor_grid,"combo_box_min_grid_style")
        self.combo_box_min_grid_style.setEnabled(0)

        group_box_minor_gridLayout.addWidget(self.combo_box_min_grid_style,4,1)
        TabPageLayout.addWidget(self.group_box_minor_grid)
        self.tab_widget_categories.insertTab(self.TabPage,QString(""))

        self.Widget3 = QWidget(self.tab_widget_categories,"Widget3")
        Widget3Layout = QGridLayout(self.Widget3,1,1,11,6,"Widget3Layout")

        self.line_edit_ratio = QLineEdit(self.Widget3,"line_edit_ratio")
        self.line_edit_ratio.setEnabled(0)

        Widget3Layout.addWidget(self.line_edit_ratio,1,1)

        self.group_box_x_axis_range = QGroupBox(self.Widget3,"group_box_x_axis_range")
        self.group_box_x_axis_range.setColumnLayout(0,Qt.Vertical)
        self.group_box_x_axis_range.layout().setSpacing(6)
        self.group_box_x_axis_range.layout().setMargin(11)
        group_box_x_axis_rangeLayout = QGridLayout(self.group_box_x_axis_range.layout())
        group_box_x_axis_rangeLayout.setAlignment(Qt.AlignTop)

        self.check_box_x_auto_scale = QCheckBox(self.group_box_x_axis_range,"check_box_x_auto_scale")
        self.check_box_x_auto_scale.setChecked(1)

        group_box_x_axis_rangeLayout.addMultiCellWidget(self.check_box_x_auto_scale,0,0,0,1)

        self.text_label_x_min = QLabel(self.group_box_x_axis_range,"text_label_x_min")
        self.text_label_x_min.setEnabled(0)

        group_box_x_axis_rangeLayout.addWidget(self.text_label_x_min,1,0)

        self.line_edit_x_min = QLineEdit(self.group_box_x_axis_range,"line_edit_x_min")
        self.line_edit_x_min.setEnabled(0)
        self.line_edit_x_min.setAlignment(QLineEdit.AlignLeft)

        group_box_x_axis_rangeLayout.addWidget(self.line_edit_x_min,1,1)

        self.text_label_x_max = QLabel(self.group_box_x_axis_range,"text_label_x_max")
        self.text_label_x_max.setEnabled(0)

        group_box_x_axis_rangeLayout.addWidget(self.text_label_x_max,2,0)

        self.line_edit_x_max = QLineEdit(self.group_box_x_axis_range,"line_edit_x_max")
        self.line_edit_x_max.setEnabled(0)

        group_box_x_axis_rangeLayout.addWidget(self.line_edit_x_max,2,1)

        Widget3Layout.addWidget(self.group_box_x_axis_range,0,0)

        self.group_box_y_axis_range = QGroupBox(self.Widget3,"group_box_y_axis_range")
        self.group_box_y_axis_range.setColumnLayout(0,Qt.Vertical)
        self.group_box_y_axis_range.layout().setSpacing(6)
        self.group_box_y_axis_range.layout().setMargin(11)
        group_box_y_axis_rangeLayout = QGridLayout(self.group_box_y_axis_range.layout())
        group_box_y_axis_rangeLayout.setAlignment(Qt.AlignTop)

        self.text_label_y_max = QLabel(self.group_box_y_axis_range,"text_label_y_max")
        self.text_label_y_max.setEnabled(0)

        group_box_y_axis_rangeLayout.addWidget(self.text_label_y_max,2,0)

        self.text_label_y_min = QLabel(self.group_box_y_axis_range,"text_label_y_min")
        self.text_label_y_min.setEnabled(0)

        group_box_y_axis_rangeLayout.addWidget(self.text_label_y_min,1,0)

        self.check_box_y_auto_scale = QCheckBox(self.group_box_y_axis_range,"check_box_y_auto_scale")
        self.check_box_y_auto_scale.setChecked(1)

        group_box_y_axis_rangeLayout.addMultiCellWidget(self.check_box_y_auto_scale,0,0,0,1)

        self.line_edit_y_min = QLineEdit(self.group_box_y_axis_range,"line_edit_y_min")
        self.line_edit_y_min.setEnabled(0)
        self.line_edit_y_min.setAlignment(QLineEdit.AlignLeft)

        group_box_y_axis_rangeLayout.addWidget(self.line_edit_y_min,1,1)

        self.line_edit_y_max = QLineEdit(self.group_box_y_axis_range,"line_edit_y_max")
        self.line_edit_y_max.setEnabled(0)

        group_box_y_axis_rangeLayout.addWidget(self.line_edit_y_max,2,1)

        Widget3Layout.addWidget(self.group_box_y_axis_range,0,1)

        self.check_box_ratio = QCheckBox(self.Widget3,"check_box_ratio")
        self.check_box_ratio.setEnabled(1)

        Widget3Layout.addWidget(self.check_box_ratio,1,0)
        self.tab_widget_categories.insertTab(self.Widget3,QString(""))
        dialog_window_configLayout.addWidget(self.tab_widget_categories)

        Layout1 = QHBoxLayout(None,0,6,"Layout1")
        Horizontal_Spacing2 = QSpacerItem(20,20,QSizePolicy.Expanding,QSizePolicy.Minimum)
        Layout1.addItem(Horizontal_Spacing2)

        self.buttonOk = QPushButton(self,"buttonOk")
        self.buttonOk.setAutoDefault(1)
        self.buttonOk.setDefault(1)
        Layout1.addWidget(self.buttonOk)

        self.buttonCancel = QPushButton(self,"buttonCancel")
        self.buttonCancel.setAutoDefault(1)
        Layout1.addWidget(self.buttonCancel)
        dialog_window_configLayout.addLayout(Layout1)

        self.languageChange()

        self.resize(QSize(544,303).expandedTo(self.minimumSizeHint()))
        self.clearWState(Qt.WState_Polished)

        self.connect(self.check_box_y_auto_scale,SIGNAL("toggled(bool)"),self.line_edit_y_max.setDisabled)
        self.connect(self.check_box_y_auto_scale,SIGNAL("toggled(bool)"),self.line_edit_y_min.setDisabled)
        self.connect(self.check_box_y_auto_scale,SIGNAL("toggled(bool)"),self.text_label_y_min.setDisabled)
        self.connect(self.check_box_y_auto_scale,SIGNAL("toggled(bool)"),self.text_label_y_max.setDisabled)
        self.connect(self.check_box_x_auto_scale,SIGNAL("toggled(bool)"),self.line_edit_x_max.setDisabled)
        self.connect(self.check_box_x_auto_scale,SIGNAL("toggled(bool)"),self.line_edit_x_min.setDisabled)
        self.connect(self.check_box_x_auto_scale,SIGNAL("toggled(bool)"),self.text_label_x_max.setDisabled)
        self.connect(self.check_box_x_auto_scale,SIGNAL("toggled(bool)"),self.text_label_x_min.setDisabled)
        self.connect(self.buttonCancel,SIGNAL("clicked()"),self.reject)
        self.connect(self.buttonOk,SIGNAL("clicked()"),self.accept)
        self.connect(self.check_box_ratio,SIGNAL("toggled(bool)"),self.line_edit_ratio.setEnabled)
        self.connect(self.check_box_x_grid_maj_enable,SIGNAL("toggled(bool)"),self.check_box_x_grid_min_enable.setEnabled)
        self.connect(self.check_box_y_grid_maj_enable,SIGNAL("toggled(bool)"),self.check_box_y_grid_min_enable.setEnabled)

        self.setTabOrder(self.line_edit_window_title,self.line_edit_x_axis_label)
        self.setTabOrder(self.line_edit_x_axis_label,self.line_edit_y_axis_label)
        self.setTabOrder(self.line_edit_y_axis_label,self.check_box_x_grid_maj_enable)
        self.setTabOrder(self.check_box_x_grid_maj_enable,self.check_box_y_grid_maj_enable)
        self.setTabOrder(self.check_box_y_grid_maj_enable,self.push_button_maj_grid_colour)
        self.setTabOrder(self.push_button_maj_grid_colour,self.spin_box_maj_grid_width)
        self.setTabOrder(self.spin_box_maj_grid_width,self.combo_box_maj_grid_style)
        self.setTabOrder(self.combo_box_maj_grid_style,self.check_box_x_grid_min_enable)
        self.setTabOrder(self.check_box_x_grid_min_enable,self.check_box_y_grid_min_enable)
        self.setTabOrder(self.check_box_y_grid_min_enable,self.push_button_min_grid_colour)
        self.setTabOrder(self.push_button_min_grid_colour,self.spin_box_min_grid_width)
        self.setTabOrder(self.spin_box_min_grid_width,self.combo_box_min_grid_style)
        self.setTabOrder(self.combo_box_min_grid_style,self.check_box_x_auto_scale)
        self.setTabOrder(self.check_box_x_auto_scale,self.line_edit_x_min)
        self.setTabOrder(self.line_edit_x_min,self.line_edit_x_max)
        self.setTabOrder(self.line_edit_x_max,self.check_box_y_auto_scale)
        self.setTabOrder(self.check_box_y_auto_scale,self.line_edit_y_min)
        self.setTabOrder(self.line_edit_y_min,self.line_edit_y_max)
        self.setTabOrder(self.line_edit_y_max,self.check_box_ratio)
        self.setTabOrder(self.check_box_ratio,self.line_edit_ratio)
        self.setTabOrder(self.line_edit_ratio,self.buttonOk)
        self.setTabOrder(self.buttonOk,self.buttonCancel)

    def languageChange(self):
        self.setCaption(self.__tr("Window Configuration"))
        self.group_box_window_labels.setTitle(self.__tr("Window labels"))
        self.text_label_y_axis_label.setText(self.__tr("Y-axis label:"))
        self.text_label_window_title.setText(self.__tr("Window title:"))
        self.text_label_x_axis_label.setText(self.__tr("X-axis label:"))
        self.tab_widget_categories.changeTab(self.Widget2,self.__tr("Labels"))
        self.group_box_major_grid.setTitle(self.__tr("Major Grid"))
        self.text_label_maj_grid_width.setText(self.__tr("Width:"))
        self.text_label_maj_grid_style.setText(self.__tr("Style:"))
        self.text_label_maj_grid_colour.setText(self.__tr("Colour:"))
        self.check_box_x_grid_maj_enable.setText(self.__tr("Enable X Major Grid"))
        self.check_box_y_grid_maj_enable.setText(self.__tr("Enable Y Major Grid"))
        self.push_button_maj_grid_colour.setText(QString.null)
        self.combo_box_maj_grid_style.clear()
        self.combo_box_maj_grid_style.insertItem(self.__tr("Solid Line"))
        self.combo_box_maj_grid_style.insertItem(self.__tr("Dashed Line"))
        self.combo_box_maj_grid_style.insertItem(self.__tr("Dotted Line"))
        self.combo_box_maj_grid_style.insertItem(self.__tr("Dash Dot Line"))
        self.combo_box_maj_grid_style.insertItem(self.__tr("Dash Dot Dot Line"))
        self.group_box_minor_grid.setTitle(self.__tr("Minor Grid"))
        self.text_label_min_grid_colour.setText(self.__tr("Colour:"))
        self.text_label_min_grid_width.setText(self.__tr("Width:"))
        self.text_label_min_grid_style.setText(self.__tr("Style:"))
        self.check_box_y_grid_min_enable.setText(self.__tr("Enable Y Minor Grid"))
        self.check_box_x_grid_min_enable.setText(self.__tr("Enable X Minor Grid"))
        self.push_button_min_grid_colour.setText(QString.null)
        self.combo_box_min_grid_style.clear()
        self.combo_box_min_grid_style.insertItem(self.__tr("Solid Line"))
        self.combo_box_min_grid_style.insertItem(self.__tr("Dashed Line"))
        self.combo_box_min_grid_style.insertItem(self.__tr("Dotted Line"))
        self.combo_box_min_grid_style.insertItem(self.__tr("Dash Dot Line"))
        self.combo_box_min_grid_style.insertItem(self.__tr("Dash Dot Dot Line"))
        self.tab_widget_categories.changeTab(self.TabPage,self.__tr("Grid (presently ignored!)"))
        self.line_edit_ratio.setText(self.__tr("1.0"))
        self.group_box_x_axis_range.setTitle(self.__tr("X-axis range"))
        self.check_box_x_auto_scale.setText(self.__tr("Enable X-axis Auto Scale"))
        self.text_label_x_min.setText(self.__tr("Minimum X value:"))
        self.line_edit_x_min.setText(self.__tr("0.0"))
        self.text_label_x_max.setText(self.__tr("Maximum X value:"))
        self.line_edit_x_max.setText(self.__tr("1000.0"))
        self.group_box_y_axis_range.setTitle(self.__tr("Y-axis range"))
        self.text_label_y_max.setText(self.__tr("Maximum Y value:"))
        self.text_label_y_min.setText(self.__tr("Minimum Y value:"))
        self.check_box_y_auto_scale.setText(self.__tr("Enable Y-axis Auto Scale"))
        self.line_edit_y_min.setText(self.__tr("0.0"))
        self.line_edit_y_max.setText(self.__tr("1000.0"))
        self.check_box_ratio.setText(self.__tr("Keep aspect ratio:"))
        self.tab_widget_categories.changeTab(self.Widget3,self.__tr("Axes"))
        self.buttonOk.setText(self.__tr("&OK"))
        self.buttonOk.setAccel(QString.null)
        self.buttonCancel.setText(self.__tr("&Cancel"))
        self.buttonCancel.setAccel(QString.null)

    def accept(self):
     QDialog.accept(self)
     
    def __tr(self,s,c = None):
        return qApp.translate("dialog_window_config",s,c)


def main(args):
    app = QApplication(args)
    demo = dialog_window_config()
    demo.show()
#   app.setMainWidget(demo)
    app.exec_loop()

# Admire
if __name__ == '__main__':
    main(sys.argv)




#!/usr/bin/env python3

#% $Id: quickref_plotter.py 6250 2008-07-11 05:55:05Z twillis $ 

#
# Copyright (C) 2002-2007
# ASTRON (Netherlands Foundation for Research in Astronomy)
# and The MeqTree Foundation
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands, seg@astron.nl
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
#
#  (c) 2013.				 (c) 2011.
#  National Research Council		 Conseil national de recherches
#  Ottawa, Canada, K1A 0R6 		 Ottawa, Canada, K1A 0R6
#
#  This software is free software;	 Ce logiciel est libre, vous
#  you can redistribute it and/or	 pouvez le redistribuer et/ou le
#  modify it under the terms of	         modifier selon les termes de la
#  the GNU General Public License	 Licence Publique Generale GNU
#  as published by the Free		 publiee par la Free Software
#  Software Foundation; either	 	 Foundation (version 3 ou bien
#  version 2 of the License, or	 	 toute autre version ulterieure
#  (at your option) any later	 	 choisie par vous).
#  version.
#
#  This software is distributed in	 Ce logiciel est distribue car
#  the hope that it will be		 potentiellement utile, mais
#  useful, but WITHOUT ANY		 SANS AUCUNE GARANTIE, ni
#  WARRANTY; without even the	 	 explicite ni implicite, y
#  implied warranty of			 compris les garanties de
#  MERCHANTABILITY or FITNESS FOR	 commercialisation ou
#  A PARTICULAR PURPOSE.  See the	 d'adaptation dans un but
#  GNU General Public License for	 specifique. Reportez-vous a la
#  more details.			 Licence Publique Generale GNU
#  					 pour plus de details.
#
#  You should have received a copy	 Vous devez avoir recu une copie
#  of the GNU General Public		 de la Licence Publique Generale
#  License along with this		 GNU en meme temps que ce
#  software; if not, contact the	 logiciel ; si ce n'est pas le
#  Free Software Foundation, Inc.	 cas, communiquez avec la Free
#  at http://www.fsf.org.		 Software Foundation, Inc. au
#						 http://www.fsf.org.
#
#  email:				 courriel:
#  business@hia-iha.nrc-cnrc.gc.ca	 business@hia-iha.nrc-cnrc.gc.ca
#
#  National Research Council		 Conseil national de recherches
#      Canada				    Canada
#  Herzberg Institute of Astrophysics	 Institut Herzberg d'astrophysique
#  5071 West Saanich Rd.		 5071 West Saanich Rd.
#  Victoria BC V9E 2E7			 Victoria BC V9E 2E7
#  CANADA					 CANADA
#

# modules that are imported
from __future__ import print_function
from __future__ import absolute_import
from __future__ import division

from qwt.qt.QtGui import QApplication, QGridLayout, QWidget
from qwt.qt.QtCore import Qt, QObject, pyqtSignal

# modules that are imported

HAS_TIMBA = False
try:
  from Timba import utils
  from Timba.Meq import meqds
  from Timba.Meq.meqds import mqs
  from MeqGUI.GUI.pixmaps import pixmaps
  from MeqGUI.GUI import widgets
  from MeqGUI.GUI.browsers import *
  from MeqGUI import Grid
  from Timba.utils import verbosity
  _dbg = verbosity(0,name='quickref_plotter');
  _dprint = _dbg.dprint;
  _dprintf = _dbg.dprintf;
  HAS_TIMBA = True
except:
  pass

#print('HAS_TIMBA = ', HAS_TIMBA)

class QuickRefPlotter(GriddedPlugin):
  """ a class to display quickref_help contents of a node in the browser """ 

  _icon = pixmaps.bars3d
  viewer_name = "QuickRef Display";
  def is_viewable (data):
    return len(data) > 0;
  is_viewable = staticmethod(is_viewable);

  def __init__(self,gw,dataitem,cellspec={},**opts):
    GriddedPlugin.__init__(self,gw,dataitem,cellspec=cellspec);
    """ Instantiate HippoDraw objects that are used to control
        various aspects of plotting, if the hippo plotter
        is instantiated.
    """
    self._rec = None;
    self._wtop = None;
    self.dataitem = dataitem
    self.layout_created = False
    self.reset_plot_stuff()

# back to 'real' work
    if dataitem and dataitem.data is not None:
      self.set_data(dataitem);

  def reset_plot_stuff(self):
    """ resets widgets to None. Needed if we have been putting
        out a message about Cache not containing results, etc
    """
    self.QTextEdit = None
    self.layout_parent = None
    self.layout = None


  def __del__(self):
    if self._window_controller:
      self._window_controller.closeAllWindows()
                                                                                           
  def wtop (self):
    """ function needed by Oleg for reasons known only to him! """
    return self._wtop;

  def create_layout_stuff(self):
    """ create grid layouts into which plotter widgets are inserted """
    if self.layout_parent is None or not self.layout_created:
      self.layout_parent = QWidget(self.wparent())
      self.layout = QGridLayout(self.layout_parent)
      self.QTextEdit = QTextEdit(self.layout_parent)
      self.layout.addWidget(self.QTextEdit, 0, 1)
      self.QTextEdit.hide()
      self.QTextEdit.setReadOnly(True)
      self.set_widgets(self.layout_parent,self.dataitem.caption,icon=self.icon())
      self.layout_created = True
    self._wtop = self.layout_parent;       

  def update_status(self, status):
     if not status is None:
       self.status_label.setText(status)

  def set_data (self,dataitem,default_open=None,**opts):
    """ This callback receives data from the meqbrowser, when the
    user has requested a QuickRef display . If there is a 
    'quickref_help' field (string) it will be shown via a
    QTextEdit widget in the browser.
    """

    if HAS_TIMBA: _dprint(3, '** in quickref_plotter:set_data callback')
    self._rec = dataitem.data;
    if HAS_TIMBA: _dprint(3, 'set_data: initial self._rec ', self._rec)
    if not isinstance(self._rec,record):
      # print '\n** self.rec not a record, but:',type(self._rec)
      pass
    elif "quickref_help" not in self._rec:
      # print '\n** self.rec does not have quickref_help field'
      pass
    else:
      qh = self._rec.quickref_help
      # print '\n** self.rec.quickref_help:',type(qh)
      if isinstance(qh,str):
        self.create_layout_stuff()
        self.QTextEdit.setText(qh)
        self.QTextEdit.show()
    return

Grid.Services.registerViewer(meqds.NodeClass(),QuickRefPlotter,priority=40)



# -*- coding: utf-8 -*-
#
#% $Id$ 
#
#
# Copyright (C) 2002-2007
# The MeqTree Foundation & 
# ASTRON (Netherlands Foundation for Research in Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc., 
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import time

from Timba import dmi
from Timba.utils import PersistentCurrier
from Timba.Meq import meqds

from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL

import six
if six.PY2:
  chr = unichr
  
chisqr = chr(0x3c7)+'<sup>2</sup>';

class SolverProgressMeter (QWidget):
  """SolverProgressMeter implements a one-line progress meter
  to track progress messages from a Solver. It is normally meant
  to be part of a status bar.
  """;
  def __init__ (self,parent):
    QWidget.__init__(self,parent);
    lo = QHBoxLayout(self);
    lo.setContentsMargins(5,0,5,0);
    lo.setSpacing(5);
    self._wlabel = QLabel(self);
    self._wlabel.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Minimum);
    self._wlabel.setAlignment(Qt.AlignLeft|Qt.AlignVCenter);
    self._wlabel.setIndent(5);
    self._wstop = QToolButton(self);
    self._wstop.setText("Enough!");
    lo.addWidget(self._wlabel);
    lo.addWidget(self._wstop);
    self._wstop.setToolTip("Press this button to stop the current solution");
    self._wstop.setToolButtonStyle(Qt.ToolButtonTextOnly);
    self._wstop.setAutoRaise(True);
    QObject.connect(self._wstop,SIGNAL("clicked()"),self.stop_solver);
    self.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Minimum);
    self._solver = None;
    self._app = None;
    self._currier = PersistentCurrier();
    self.curry = self._currier.curry;
    self.xcurry = self._currier.xcurry;
    self._hidetimer = QTimer(self);
    # iterations-per-second stats -- kept separately by solver name
    # in case of cuncurrent solvers
    self._ips = {};
    QObject.connect(self._hidetimer,SIGNAL("timeout()"),self._timed_reset);
    
  def connect_app_signals (self,app):
    """connects standard app signals to appropriate methods."""
    self._app = app;
    QObject.connect(app,PYSIGNAL("solver.begin"),self.xcurry(self.solver_begin,_argslice=slice(1,2)));
    QObject.connect(app,PYSIGNAL("solver.iter"),self.xcurry(self.solver_iter,_argslice=slice(1,2)));
    QObject.connect(app,PYSIGNAL("solver.end"),self.xcurry(self.solver_end,_argslice=slice(1,2)));
    QObject.connect(app,PYSIGNAL("isConnected()"),self.xcurry(self.reset));
    
  def _show (self,secs=60):
    """shows meter, and starts timer to hide after the specified number of
    seconds""";
    self._hidetimer.setSingleShot(True)
    self._hidetimer.start(secs*1000);
    self._wlabel.show();
    self.show();
  
  def solver_begin (self,rec):
    """processes solver.begin record. Usually connected to a Solver.Begin signal""";
    self._app.statusbar.clearMessage();
    if rec.num_tiles > 1:
      msg = "<b>%(node)s</b> p:%(num_spids)d t:%(num_tiles)d u:<b>%(num_unknowns)d</b>"%rec;
    else:
      msg = "<b>%(node)s</b> p:%(num_spids)d u:<b>%(num_unknowns)d</b>"%rec;
    self._solver = rec.node;
    self._wlabel.setText("<nobr>"+msg+"</nobr>");
    self._wstop.setText("Enough!");
    self._wstop.setEnabled(True);
    self._show();
    self._ips.pop(rec.node,None);
    
  def solver_iter (self,rec):
    """processes solver.iter record. Usually connected to a Solver.Iter signal""";
    # form basic message
    if 'chi_0' in rec:  # new-style solver reports chi value
      msg = ("<b>%(node)s</b> i<b>%(iterations)d</b> "+chisqr+":<b>%(chi_0).3g</b> rank:<b>%(rank)d</b>/%(num_unknowns)d ")%rec;
    else: # old-style meter reports fit value only
      msg = "<b>%(node)s</b> i<b>%(iterations)d</b> fit:<b>%(fit).3g</b> rank:<b>%(rank)d</b>/%(num_unknowns)d "%rec;
    if rec.num_tiles > 1:
      msg += "c:%(num_converged)d/%(num_tiles)d"%rec;
    self._solver = rec.node;
    # start the iteration timer at iteration 1, or at a later iteration
    # if we somehow missed iteration 1
    (time0,iter0) = self._ips.get(rec.node,(None,None));
    if rec.iterations == 1 or time0 is None:
      self._ips[rec.node] = (time.time(),rec.iterations);
    # else update label
    elif time0 is not None:
      niter = rec.iterations - iter0;
      dt = time.time() - time0;
      if dt and niter:
        msg += " (%.3g sec/iter)" % (dt/niter); 
    self._wlabel.setText("<nobr>"+msg+"</nobr>");
    self._wstop.setText("Enough!");
    self._wstop.setEnabled(True);
    self._show();
    
  def solver_end (self,rec):
    """processes solver.end record. Usually connected to a Solver.End signal""";
    if not hasattr(rec,'converged'):
      self._wlabel.setText("");
      self._wstop.setText("");
      self._solver = None;
    else:
      if rec.converged:
        self._wstop.setText("ok");
        color="darkgreen";
      else:
        self._wstop.setText("n/c");
        color="red";
      rec.final_iter = "<font color=\"%s\">i<b>%d</b></font>"%(color,rec.iterations);
      if 'chi_0' in rec:  # new-style solver reports chi value
        msg = ("<b>%(node)s</b> %(final_iter)s "+chisqr+":<b>%(chi_0).3g</b> rank:<b>%(rank)d</b>/%(num_unknowns)d ")%rec;
      else: # old-style meter reports fit value only
        msg = "<b>%(node)s</b> %(final_iter)s fit:<b>%(fit).3g</b> rank:<b>%(rank)d</b>/%(num_unknowns)d "%rec;
      if rec.num_tiles > 1:
        msg += "c:%(num_converged)d/%(num_tiles)d"%rec;
  #    if not rec.converged:
  #      msg += " <b><font color=\"red\">N/C</font><b>";
      self._wlabel.setText("<nobr>"+msg+"</nobr>");
    self._show(20);
    self._solver = None;
    self._wstop.setEnabled(False);
    
  def _timed_reset (self):
    self.reset();
    
  def reset (self):
    """resets and hides meter."""
    self._ips = {};
    self._wlabel.setText(""); 
    self._wlabel.hide();
    self.hide();
    self._hidetimer.stop();
    self._solver = None;
    
  def enable_stop_button (self):
    self._wstop.setText("Enough!");
    self._wstop.setEnabled(True);
    
  def disable_stop_button (self,message):
    self._wstop.setText(message);
    self._wstop.setEnabled(False);

  def stop_solver (self):
    if not self._solver:
      return;
    self.disable_stop_button("stopping...");
    meqds.mqs().setnodestate(self._solver,dmi.record(interrupt_solution=True),sync=False);
        



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#
#% $Id: tdlgui.py 6822 2009-03-03 15:13:52Z oms $
#
#
# Copyright (C) 2002-2007
# The MeqTree Foundation &
# ASTRON (Netherlands Foundation for Research in Astronomy)
# P.O.Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>,
# or write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

import Timba
from MeqGUI import Grid, GUI, Plugins
from Timba.dmi import *
from Timba.utils import *
from MeqGUI.GUI import browsers
from MeqGUI.GUI.pixmaps import pixmaps
from MeqGUI.GUI import app_proxy_gui
from Timba.Meq import meqds
from Timba import TDL
import Timba.TDL.Settings
import Timba.TDL.Compile
from Timba.TDL import TDLOptions

from PyQt4.Qt import *
from Kittens.widgets import PYSIGNAL,BusyIndicator

import imp
import sys
import re
import traceback
import os
import os.path
import tempfile

_dbg = verbosity(0,name='tdlgui');
_dprint = _dbg.dprint;
_dprintf = _dbg.dprintf;

def _file_mod_time (path):
  try:
    return os.stat(path).st_mtime;
  except IOError:
    return None;

# this is information about ourselves
_MODULE_FILENAME = Timba.utils.extract_stack()[-1][0];
_MODULE_DIRNAME = os.path.dirname(_MODULE_FILENAME);

from .TDLErrorFloat import TDLErrorFloat
from .TDLOptionsDialog import TDLOptionsDialog

class TDLEditor (QFrame,PersistentCurrier):
  SupportsLineNumbers = False;

  # a single editor always has the focus
  current_editor = None;

  # flag: sync to external editor
  external_sync = True;
  def set_external_sync (value):
    global _external_sync;
    _external_sync = value;
  set_external_sync = staticmethod(set_external_sync);

  def __init__ (self,parent,close_button=False,error_window=None):
    QFrame.__init__(self,parent);
    self._enabled = True;
    toplo = QVBoxLayout(self);
    toplo.setContentsMargins(0,0,0,0);
    toplo.setSpacing(0);

    self.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding);
    splitter = QSplitter(Qt.Vertical,self);
    toplo.addWidget(splitter);
    splitter.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding);
    splitter.setChildrenCollapsible(False);

    # figure out our app_gui parent
    self._appgui = app_proxy_gui.appgui(parent);

    # create an editor box
    editor_box = QFrame(splitter);
    lo = QVBoxLayout(editor_box);
    lo.setContentsMargins(0,0,0,0);
    lo.setSpacing(0);

    # find main window to associate our toolbar with
    mainwin = parent;
    while mainwin and not isinstance(mainwin,QMainWindow):
      mainwin = mainwin.parent();

    self._toolbar = QToolBar(mainwin or self);
    lo.addWidget(self._toolbar);
    self._toolbar.setIconSize(QSize(16,16));

    #### populate toolbar

    # Exec button and menu
    self._tb_tdlexec = QToolButton(self._toolbar);
    self._toolbar.addWidget(self._tb_tdlexec);
    self._tb_tdlexec.setIcon(pixmaps.gear.icon());
    self._tb_tdlexec.setText("Exec");
    self._tb_tdlexec.setToolButtonStyle(Qt.ToolButtonTextBesideIcon);
    self._tb_tdlexec.setToolTip("Accesses run-time options & jobs defined by this TDL script");
    self._tb_tdlexec.hide();

    jobs = self._tdlexec_menu = TDLOptionsDialog(self);
    jobs.setWindowTitle("TDL Jobs & Runtime Options");
    jobs.setWindowIcon(pixmaps.gear.icon());
    jobs.hide();
    QObject.connect(self._tb_tdlexec,SIGNAL("clicked()"),jobs.exec_);

    # save menu and button
    self._tb_save = QToolButton(self._toolbar);
    self._toolbar.addWidget(self._tb_save);
    self._tb_save.setIcon(pixmaps.file_save.icon());
    self._tb_save.setToolTip("Saves script. Click on the down-arrow for other options.");

    savemenu = QMenu(self);
    self._tb_save.setMenu(savemenu);
    self._tb_save.setPopupMode(QToolButton.MenuButtonPopup);
    self._tb_save._modified_color = QColor("yellow");
    qa_save = savemenu.addAction(pixmaps.file_save.icon(),"&Save script",self._save_file);
    qa_save.setShortcut(Qt.ALT+Qt.Key_S);
    QObject.connect(self._tb_save,SIGNAL("clicked()"),self._save_file);
    qa_save_as = savemenu.addAction(pixmaps.file_save.icon(),"Save script &as...",
                                     self.curry(self._save_file,save_as=True));
    qa_revert = self._qa_revert = savemenu.addAction("Revert to saved",self._revert_to_saved);

    # run menu and button

    self._qa_run = self._toolbar.addAction(pixmaps.blue_round_reload.icon(),
                              "&Save & reload",self._import_main_file);
    self._qa_run.setShortcut(Qt.ALT+Qt.Key_R);

    # Compile-time options and menu
    self._tb_opts = QToolButton(self._toolbar);
    self._toolbar.addWidget(self._tb_opts);
    self._tb_opts.setIcon(pixmaps.wrench.icon());
    self._tb_opts.setText("Options");
    self._tb_opts.setToolButtonStyle(Qt.ToolButtonTextBesideIcon);
    self._tb_opts.setToolTip("Accesses compile-time options for this TDL script");
    # self._tb_opts.hide();

    opts = self._options_menu = TDLOptionsDialog(self,ok_label="Compile",ok_icon=pixmaps.blue_round_reload);
    opts.setWindowTitle("TDL Compile-time Options");
    opts.setWindowIcon(pixmaps.wrench.icon());
    QObject.connect(opts,PYSIGNAL("accepted()"),self._compile_main_file);
    QObject.connect(TDLOptions.OptionObject,SIGNAL("mandatoryOptionsSet"),self.mark_mandatory_options_set);
    opts.hide();
    QObject.connect(self._tb_opts,SIGNAL("clicked()"),opts.show);
    # cross-connect the rereshProfiles() signals, so that dialogs can update each other's
    # profile menus
    QObject.connect(self._options_menu,PYSIGNAL("refreshProfiles()"),self._tdlexec_menu.refresh_profiles);
    QObject.connect(self._tdlexec_menu,PYSIGNAL("refreshProfiles()"),self._options_menu.refresh_profiles);

    self._toolbar.addSeparator();

    # cursor position indicator
    self._poslabel = QLabel(self._toolbar);
    #wa = QWidgetAction(self._toolbar);
    #wa.setDefaultWidget(self._poslabel);
    #self._toolbar.addAction(wa);
    #self._toolbar.addWidget(self._poslabel);
    self._toolbar.addWidget(self._poslabel);
    width = self._poslabel.fontMetrics().width("L:999 C:999");
    self._poslabel.setMinimumWidth(width);
    self._poslabel.setText("L:0 C:0");

    # filename indicator
    self._pathlabel = QLabel(self._toolbar);
    #wa = QWidgetAction(self._toolbar);
    #wa.setDefaultWidget(self._pathlabel);
    self._toolbar.addWidget(self._pathlabel);
    #self._toolbar.addAction(wa);
    self._pathlabel.show();
    self._pathlabel.setAlignment(Qt.AlignRight|Qt.AlignVCenter);
    self._pathlabel.setIndent(10);
    self._pathlabel.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Minimum);
    if close_button:
      if not isinstance(close_button,QIcon):
        close_button = pixmaps.remove.icon();
      self._qa_close = self._toolbar.addAction(close_button,"&Close file",self._file_closed);
      self._qa_close.setShortcut(Qt.ALT+Qt.Key_W);
    self._pathlabel.setText("(no file)");

    #### add editor window

    self._editor = editor = QTextEdit(editor_box);
    lo.addWidget(self._editor);
    editor.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Expanding);
    editor.setAcceptRichText(False);
    editor.setLineWrapMode(QTextEdit.NoWrap);
    self._document = QTextDocument(self);
    editor.setDocument(self._document);
    QObject.connect(self._document,SIGNAL("modificationChanged(bool)"),self._text_modified);
    QObject.connect(self._editor,SIGNAL("cursorPositionChanged()"),self._display_cursor_position);
    # QObject.connect(self._editor,SIGNAL("textChanged()"),self._clear_transients);

    # add character formats for error display
    self._format_error = QTextCharFormat(self._editor.currentCharFormat());
    self._format_error.setBackground(QBrush(QColor("lightpink")));
    self._format_suberror = QTextCharFormat(self._editor.currentCharFormat());
    self._format_suberror.setBackground(QBrush(QColor("lightgrey")));
    self._format_current_error = QTextCharFormat(self._editor.currentCharFormat());
    self._format_current_error.setBackground(QBrush(QColor("orangered")));

    # add message window
    self._message_box = QFrame(editor_box);
    lo.addWidget(self._message_box);
    self._message_box.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Preferred);
    self._message_box.setFrameStyle(QFrame.Panel+QFrame.Sunken);
    self._message_box.setLineWidth(2);
    mblo = QVBoxLayout(self._message_box);
    mblo.setContentsMargins(0,0,0,0);
    msgb1 = QHBoxLayout();
    mblo.addLayout(msgb1);
    msgb1.setContentsMargins(0,0,0,0);
    msgb1.setSpacing(0);
    self._message_icon = QToolButton(self._message_box);
    msgb1.addWidget(self._message_icon);
    self._message = QLabel(self._message_box);
    msgb1.addWidget(self._message);
    self._message.setTextFormat(Qt.RichText);
    self._message.setWordWrap(True);
    self._message.setMargin(0);
    self._message.setSizePolicy(QSizePolicy.Expanding,QSizePolicy.Preferred);
    # self._message_icon.setAlignment(Qt.AlignTop);
    # self._message_icon.setMargin(4);
    self._message_icon.setAutoRaise(True);
    self._message_icon.setSizePolicy(QSizePolicy.Fixed,QSizePolicy.Fixed);
    QObject.connect(self._message_icon,SIGNAL("clicked()"),self.clear_message);
    self._message_icon.setToolTip("Click here to clear the message");
    self._message_widgets = [];
    self._message_transient = False;

    # figure out if we already have an error box to attach to
    self._error_window = error_window or getattr(parent,'_tdlgui_error_window',None);
    if self._error_window:
      #self._resize_errwin = False;
      pass;
    else:
      # otherwise create an error floater
      self._error_window = TDLErrorFloat(parent);
      setattr(parent,'_tdlgui_error_window',self._error_window);
      # self._resize_errwin = True;
    QObject.connect(self._error_window,PYSIGNAL("hasErrors()"),self._reset_errors);
    QObject.connect(self._error_window,PYSIGNAL("showError()"),self.show_error);

    # set filename
    self._filename = None;       # "official" path of file (None if new script not yet saved)
    self._mainfile = None;       # if not None, then we're "slaved" to a main file (see below)
    self._file_disktime = None;  # modtime on disk when file was loaded
    self._basename = None;
    self._modified = False;
    self._closed = False;
    self._error_at_line = {};
    self._is_tree_in_sync = True;
    self._pub_nodes = None;

  def __del__ (self):
    self.has_focus(False);

  def disable_editor (self):
    """Called before disabling the editor, as on some versions of PyQt
    the object is not destroyed properly and keeps receving signals""";
    self._enabled = False;
    QObject.disconnect(TDLOptions.OptionObject,SIGNAL("mandatoryOptionsSet"),self.mark_mandatory_options_set);

  def show_compile_options (self):
    self._options_menu.show();

  def mark_mandatory_options_set (self,enabled):
    self._options_menu.enableOkButton(enabled);

  def show_runtime_options (self):
    self._tdlexec_menu.show();

  def tree_is_in_sync (self,sync=True):
    """Tells the editor wheether the current tree is in sync with the content of the script.
    This is indicated by a visual cue on the toolbar.
    """;
    if sync:
      self._tb_tdlexec.setIcon(pixmaps.gear.icon());
      self._tb_tdlexec.setToolTip("Access run-time options & jobs defined by this TDL script");
    else:
      self._tb_tdlexec.setIcon(pixmaps.exclaim_yellow_warning.icon());
      self._tb_tdlexec.setToolTip("""Access run-time options & jobs defined by this TDL script.
Warning! You have modified the script since it was last compiled, so the tree may be out of date.""");

  def _file_closed (self):
    self.emit(PYSIGNAL("fileClosed()"),self);

  def hideEvent (self,ev):
    self.emit(PYSIGNAL("hidden()"));
    self.emit(PYSIGNAL("visible()"),False);
    return QFrame.hideEvent(self,ev);

  def showEvent (self,ev):
    self.emit(PYSIGNAL("shown()"));
    self.emit(PYSIGNAL("visible()"),True);
    return QFrame.showEvent(self,ev);

  def hide_jobs_menu (self,dum=False):
    if self._closed:
      return;
    self._tb_tdlexec.hide();
    self.clear_message();

  def show_line_numbers (self,show):
    pass;

  def show_run_control (self,show=True):
    if self._closed:
      return;
    self._qa_run.setVisible(show);

  def enable_controls (self,enable=True):
    if self._closed:
      return;
    self._qa_run.setEnabled(enable);
    self._tb_tdlexec.setEnabled(enable);
    if not enable:
      self.clear_message();

  def disable_controls (self,disable=True):
    if self._closed:
      return;
    self._qa_run.setDisabled(disable);
    self._tb_tdlexec.setDisabled(disable);
    if disable:
      self.clear_message();

  def get_filename (self):
    return self._filename;
  def get_mainfile (self):
    return self._mainfile;

  def _import_main_file (self):
    # self._tb_opts.setOn(False);
    self.clear_errors();
    if self._document.isModified():
      self._save_file();
    self.emit(PYSIGNAL("importFile()"),self,self._mainfile or self._filename);

  def _compile_main_file (self):
    # self._tb_opts.setOn(False);
    self.clear_errors();
    if self._document.isModified():
      self._save_file();
    self.emit(PYSIGNAL("compileFile()"),self,self._mainfile or self._filename);

  def _clear_transients (self):
    """if message box contains a transient message, clears it""";
    if self._message_transient:
      self.clear_message();

  def _display_cursor_position (self):
    cursor = self._editor.textCursor();
    pos = cursor.position();
    col = cursor.columnNumber();
    line = cursor.blockNumber();
    self._poslabel.setText("L:<b>%d</b> C:<b>%d</b>" % (line+1,col+1));
    self._poslabel.repaint();


  def _text_modified (self,mod=True):
    self._modified = mod;
    self.emit(PYSIGNAL("textModified()"),self,bool(mod));
    self._tb_save.setAutoRaise(not mod);
    self._qa_revert.setEnabled(mod);
    #if mod:
      #self._tb_save.setBackgroundRole(QPalette.ToolTipBase);
    #else:
      #self._tb_save.setBackgroundRole(QPalette.Button);
    if self._filename:
      label = '<b>' + self._basename + '</b>';
      self._pathlabel.setToolTip(self._filename);
    else:
      label = '';
      self._pathlabel.setToolTip('');
    if self._readonly:
      label = '[r/o] ' + label;
    if mod:
      self._clear_transients();
      label = '[mod] ' + label;
    self._pathlabel.setText(label);

  def clear_message (self):
    # traceback.print_stack();
    self._message_box.hide();
    self._message.setText('');
    self._message_icon.setText('');
    self._message_transient = False;
    if self._message_widgets:
      dum = QWidget();
      for w in self._message_widgets:
        w.reparent(dum);
      self._message_widgets = [];

  def show_message (self,msg,error=False,icon=None,transient=False):
    """Shows message in box.
    If icon is not None, overrides standard icon.
    If error=True, uses error icon (icon overrides this).
    If transient=True, message will be cleared when text is edited.
    """;
    self._message.setText(msg);
    if not icon:
      if error:
        icon = pixmaps.red_round_cross.icon();
      else:
        icon = pixmaps.info_blue_round.icon();
    self._message_icon.setIcon(icon);
    self._message_box.show();
    self._message_transient = transient;
    self.clear_errors();

  def messagebox ():
    return self._message_box;

  def add_message_widget (self,widget):
    self._mblo.addWidget(widget);
    self._message_widgets.append(widget);

  def clear_errors (self):
    self._error_window.clear_errors(emit_signal=True);

  def _find_block_by_linenumber (self,line):
    block = block0 = self._document.begin();
    nblock = 1;
    while block and nblock < line:
      block0 = block;
      block = next(block0);
      nblock += 1;
    return block or block0;

  def _reset_errors (self,nerr):
    """helper method, resets error markers and such. Usually tied to a hasErrors() signal
    from an error window""";
    if not self._enabled:
      return;
    self._error_at_line = {};
    self._error_selections = [];
    self._current_error_selection = QTextEdit.ExtraSelection();
    nerr_local = 0;
    if nerr:
      error_locations = self._error_window.get_error_locations();
      for err_num,filename,line,column,suberror in error_locations:
        if filename == self._filename:
          nerr_local += 1;
          # make cursor corresponding to line containing the error
          cursor = QTextCursor(self._find_block_by_linenumber(line));
          cursor.select(QTextCursor.LineUnderCursor);
          # make selection object
          qsel = QTextEdit.ExtraSelection();
          qsel.cursor = cursor;
          if suberror:
            qsel.format = self._format_suberror;
          else:
            qsel.format = self._format_error;
          # insert into error_at_line list
          self._error_at_line[line-1] = len(self._error_selections),cursor;
          # append to list of error selections
          self._error_selections.append(qsel);
      self._editor.setExtraSelections(self._error_selections);
    else:
      self._editor.setExtraSelections([]);
    self.emit(PYSIGNAL("hasErrors()"),self,nerr_local);

  def show_error (self,err_num,filename,line,column):
    """Shows error at the given position, but only if the filename matches.
    Can be directly connected to a showError() signal from an error window""";
    if filename == self._filename:
      errnum,cursor = self._error_at_line.get(line-1,(None,None));
      if cursor:
        # change selections
        self._current_error_selection.cursor = cursor;
        self._current_error_selection.format = self._format_current_error;
        sels = self._error_selections[:errnum] + self._error_selections[(errnum+1):];
        sels.append(self._current_error_selection);
        self._editor.setExtraSelections(sels);
        # move current cursor
        cursor = self._editor.textCursor();
        # scroll to line-1 and line+1 to make sure line is fully visible
        if line > 1:
          cursor.setPosition(self._find_block_by_linenumber(line-2).position() + column);
          self._editor.setTextCursor(cursor);
          self._editor.ensureCursorVisible();
        if line < self._document.blockCount():
          cursor.setPosition(self._find_block_by_linenumber(line).position() + column);
          self._editor.setTextCursor(cursor);
          self._editor.ensureCursorVisible();
        # finally, scroll to line
        cursor.setPosition(self._find_block_by_linenumber(line).position() + column);
        self._editor.setTextCursor(cursor);
        self._editor.ensureCursorVisible();


  def sync_external_file (self,filename=None,ask=True):
    #
    # NB: problem because it resets the errors
    filename = filename or self._filename;
    filetime = _file_mod_time(filename);
    if not filetime or filetime == self._file_disktime:
      return True;  # in sync
    if not ask:
      res = QMessageBox.Discard;
    else:
      res = QMessageBox.warning(self,"TDL file changed",
        """<p><tt>%s</tt> has been modified by another program.
        Would you like to save your changes and overwrite the version on disk,
	discard your changes and revert to the version on disk, or cancel the operation?"""
        % (filename,),QMessageBox.Save|QMessageBox.Discard|QMessageBox.Cancel);
    if res == QMessageBox.Cancel:
      return None;
    elif res == QMessageBox.Discard:
      if filename != self._filename:
        self.load_file(filename,mainfile=self._mainfile);
      else:
        self.reload_file();
    return True;  # in sync

  def _save_file (self,filename=None,text=None,force=False,save_as=False):
    """Saves text. If force=False, checks modification times in case
    the file has been modified by another program.
    If force=True, saves unconditionally.
    If no filename is known, asks for one.
    Returns True if file was successfully saved, else None.""";
    filename = filename or self._filename;
    if filename and not save_as:
      if not force:
        if not self.sync_external_file(filename=filename,ask=True):
          return None;
    else: # no filename, ask for one
      try:
        dialog = self._save_as_dialog;
        dialog.setDirectory(dialog.directory());  # hopefully this rescan the directory
      except AttributeError:
        self._save_as_dialog = dialog = QFileDialog(self,"Saved TDL Script");
        dialog.resize(800,dialog.height());
        dialog.setMode(QFileDialog.AnyFile);
        dialog.setNameFilters(["TDL scripts (*.tdl *.py)","All files (*.*)"]);
        dialog.setViewMode(QFileDialog.Detail);
      if dialog.exec_() != QDialog.Accepted:
        return None;
      filename = str(dialog.selectedFiles()[0]);
    # save the file
    if text is None:
      text = str(self._editor.document().toPlainText());
    try:
      outfile = open(filename,"w").write(text);
    except IOError:
      (exctype,excvalue,tb) = sys.exc_info();
      _dprint(0,'exception',sys.exc_info(),'saving TDL file',filename);
      self.show_message("""<b>Error writing <tt>%s</tt>:
        <i>%s (%s)</i></b>""" % (filename,excvalue,exctype.__name__),
        error=True,transient=True);
      return None;
    # saved successfully, update stuff
    self._filename = filename;
    self._qa_revert.setEnabled(True);
    self._basename = os.path.basename(filename);
    self._file_disktime = _file_mod_time(filename);
    self._document.setModified(False);
    self.emit(PYSIGNAL("fileSaved()"),self,filename);
    return self._filename;

  def close (self):
    self._closed = True;

  def confirm_close (self):
    if self._document.isModified():
      res = QMessageBox.warning(self,"TDL file modified",
        """Save modified file <p><tt>%s</tt>?</p>"""
        % (self._filename or "",),QMessageBox.Save|QMessageBox.Discard|QMessageBox.Cancel);
      if res == QMessageBox.Cancel:
        return False;
      if res == QMessageBox.Save:
        if not self._save_file():
          return False;
    self.close();
    return True;

  def _revert_to_saved (self,force=False):
    if not self._filename:
      return;
    if not force:
      if QMessageBox.question(self,"Revert to saved",
	  """Revert to saved version of <p><tt>%s</tt>?"""%(self._filename,),
	  QMessageBox.Ok|QMessageBox.Cancel) == QMessageBox.Cancel:
        return;
    self.load_file(self._filename,mainfile=self._mainfile);

  def _add_menu_label (self,menu,label):
    tlab = QLabel("<b>"+label+"</b>",menu);
    tlab.setAlignment(Qt.AlignCenter);
    tlab.setFrameShape(QFrame.ToolBarPanel);
    tlab.setFrameShadow(QFrame.Sunken);
    menu.insertItem(tlab);

  def has_compile_options (self):
    return self._options_menu.treeWidget().topLevelItemCount();

  def has_runtime_options (self):
    return self._tdlexec_menu.treeWidget().topLevelItemCount();

  def import_content (self,force=False,show_options=False):
    """imports TDL module but does not run _define_forest().
    Depending on autosync/modified state, asks to save or revert.
    If module is already imported, does nothing, unless force=True,
    in which case it imports unconditionally.
    If do_compile=True, proceeds to show compile-time options on success,
    or to compile directly if there are no options
    Return value:
      True on successful import
      None if cancelled by user.
    Import errors are posted to the error dialog.
    """;
    # save list of publishers (since forest will be cleared on import)
    if self._pub_nodes is None:
      if TDL.Compile.last_imported_file() == self._filename:
        self._pub_nodes = [ node.name for node in meqds.nodelist.iternodes() if node.is_publishing() ];
        _dprint(2,"last imported file matches, saving",len(self._pub_nodes),"publishers")
      else:
        self._pub_nodes = [];
        _dprint(2,"last imported file doesn't match:",TDL.Compile.last_imported_file(),self._filename)
    _dprint(1,self._filename,"importing");
    self.clear_message();
    self.clear_errors();
    self._options_menu.hide();
    self._tb_tdlexec.hide();
    self._tdlexec_menu.hide();
    # change the current directory to where the file is
    # os.chdir(os.path.dirname(self._filename));
    # The Python imp module expects text to reside in a disk file, which is
    # a pain in the ass for us if we're dealing with modified text or text
    # entered on-the-fly. So, either save or sync before proceeding
    global _external_sync;
    if self._document.isModified() or not self._filename:
      if not self._save_file():
        return None;
    else:
      if not self.sync_external_file(ask=False):
        return None;
    # if we already have an imported module and disk file hasn't changed, skip
    # the importing step
    busy = BusyIndicator();
    if force or self._tdlmod is None or self._tdlmod_filetime == self._file_disktime:
      # reset data members
      _dprint(2,self._filename,"emitting signal for 0 compile-time options");
      self.emit(PYSIGNAL("hasCompileOptions()"),self,0);
      self._options_menu.hide();
      self._options_menu.clear();
      self._tdlmod = None;
      # get text from editor
      tdltext = str(self._document.toPlainText());
      try:
        tdlmod,tdltext = TDL.Compile.import_tdl_module(self._filename,tdltext);
      # catch import errors
      except TDL.CumulativeError as value:
        _dprint(0,"caught cumulative error, length",len(value.args));
        self._error_window.set_errors(value.args,message="TDL import failed");
        busy = None;
        return None;
      except Exception as value:
        _dprint(0,"caught other error, traceback follows");
        traceback.print_exc();
        self._error_window.set_errors([value],message="TDL import failed");
        busy = None;
        return None;
      # remember module and nodescope
      self._tdlmod = tdlmod;
      self._tdltext = tdltext;
      self._tdlmod_filetime = self._file_disktime;
      # build options menu from compile-time options
      opt_tw = self._options_menu.treeWidget();
      opts = TDLOptions.get_compile_options();
      if opts:
        # add options
        try:
          TDLOptions.populate_option_treewidget(opt_tw,opts);
        except Exception as value:
          _dprint(0,"error setting up TDL options GUI");
          traceback.print_exc();
          self._error_window.set_errors([value],message="Error setting up TDL options GUI");
          busy = None;
          return None;
        # self._tb_opts.show();
        _dprint(2,self._filename,"emitting signal for",len(opts),"compile-time options");
        self.emit(PYSIGNAL("hasCompileOptions()"),self,len(opts));
    # success, show options or compile
    if show_options and self.has_compile_options():
      self._options_menu.adjustSizes();
      self._options_menu.show();
    busy = None;
    return True;

  def compile_content (self):
    # import content first, and return if failed
    if not self.import_content(force=True):
      return None;
    _dprint(1,self._filename,"compiling forest");
    # clear predefined functions
    self._tb_tdlexec.hide();
    # try the compilation
    busy = BusyIndicator();
    try:
      (_tdlmod,ns,msg) = \
	TDL.Compile.run_forest_definition(
	    meqds.mqs(),self._filename,self._tdlmod,self._tdltext,
	    parent=self,wait=False);
    # catch compilation errors
    except TDL.CumulativeError as value:
      _dprint(0,"caught cumulative error, length",len(value.args));
      self._error_window.set_errors(value.args,message="TDL import failed");
      busy = None;
      return None;
    except Exception as value:
      _dprint(0,"caught other error, traceback follows");
      traceback.print_exc();
      self._error_window.set_errors([value]);
      busy = None;
      return None;
    # refresh the nodelist
    meqds.request_nodelist(sync=True);
    # restore publishing nodes
    for name in (self._pub_nodes or []):
      if name in ns.AllNodes():
        _dprint(2,"reenabling publishing of",name);
        meqds.enable_node_publish_by_name(name,sync=True,get_state=True);
      else:
        _dprint(2,"not reenabling publishing of",name,", as it no longer exists?");
    # clear publisher list so it's re-saved next time in import_content()
    self._pub_nodes = None;
    ### NB: presume this all was successful for now
    # insert node scope into TDL module
    setattr(_tdlmod,'_tdl_nodescope',ns);

    # does the script define an explicit job list?
    joblist = getattr(_tdlmod,'_tdl_job_list',[]);
    if not joblist:
      joblist = [];
      # try to build it from implicit function names
      for (name,func) in _tdlmod.__dict__.items():
        if name.startswith("_tdl_job_") and callable(func) and not TDLOptions.is_jobfunc_defined(func):
          joblist.append(func);
    # does the script define a testing function?
    testfunc = getattr(_tdlmod,'_test_forest',None);
    if not callable(testfunc):
      testfunc = getattr(_tdlmod,'test_forest',None);
      if callable(testfunc):
        res = QMessageBox.warning(self,"Deprecated method",
          """Your script contains a test_forest() method. This is deprecated
          and will be disabled in the future. Please rename it to
          _test_forest().
          """,
          QMessageBox.Ok);
    if callable(testfunc):
      joblist.append(testfunc);
    
    from past.builtins import cmp
    from functools import cmp_to_key
    joblist.sort(key=cmp_to_key(lambda a, b: cmp(str(a), str(b))));

    # create list of job actions
    opts = TDLOptions.get_runtime_options();
    self._tdlexec_menu.clear();
    if joblist or opts:
      if opts:
        self._job_executor = curry(self.execute_tdl_job,_tdlmod,ns);
        ## new style:
        try:
          TDLOptions.populate_option_treewidget(self._tdlexec_menu.treeWidget(),opts,executor=self._job_executor);
        except Exception as value:
          _dprint(0,"error setting up TDL options GUI");
          traceback.print_exc();
          self._error_window.set_errors([value],message="Error setting up TDL options GUI");
          busy = None;
          return None;
      if joblist:
        for func in joblist:
          name = re.sub("^_tdl_job_","",func.__name__);
          name = name.replace('_',' ');
          self._tdlexec_menu.addAction(name,
              curry(self.execute_tdl_job,_tdlmod,ns,func,name,func.__name__),
              icon=pixmaps.gear);
      self.emit(PYSIGNAL("hasRuntimeOptions()"),self,True);
      self._tdlexec_menu.adjustSizes();
      self._tb_tdlexec.show();
    else:
      self.emit(PYSIGNAL("hasRuntimeOptions()"),self,False);
      self._tb_tdlexec.hide();

    if joblist:
      msg += " %d predefined function(s) available, please use the Exec menu to run them." % (len(joblist),);

    self.show_message(msg,transient=True);
    busy = None;
    return True;

  def execute_tdl_job (self,_tdlmod,ns,func,name,job_id):
    """executes a predefined TDL job given by func""";
    self._tdlexec_menu.hide();
    try:
      # log job
      TDLOptions.dump_log("running TDL job '%s' (%s)"%(name,("job id '%s'"%job_id if job_id else "no job id")));
      busy = BusyIndicator();
      func(meqds.mqs(),self);
      # no errors, so clear error list, if any
      self.clear_errors();
      self.show_message("TDL job '"+name+"' executed.",transient=True);
      busy = None;
    except:
      (etype,exc,tb) = sys.exc_info();
      _dprint(0,'exception running TDL job',func.__name__);
      traceback.print_exception(etype,exc,tb);
      # use TDL add_error() to process the error, since this automatically
      # adds location information. However, we want to remove ourselves
      # from the stack traceback first
      tb = traceback.extract_tb(tb);
      # pop everything leading up to our filename
      while tb[0][0] != _MODULE_FILENAME:
        tb.pop(0);
      # pop frame with our filename, this should leave only TDL-code frames
      tb.pop(0);
      ns.AddError(exc,tb,error_limit=None);
      msg = "TDL job '"+name+"' failed";
      self._error_window.set_errors(ns.GetErrors(),message=msg);
      self.emit(PYSIGNAL("showEditor()"),self);
      busy = None;

  def get_jobs_popup (self):
    return self._tdlexec_menu;

  def get_options_popup (self):
    return self._options_menu;

  def _set_mainfile (self,mainfile):
    """adjusts GUI controls based on whether we are a mainfile or not""";
    self._mainfile = mainfile;
    self._mainfile_base = mainfile and os.path.basename(mainfile);
    # if we have a mainfile, add extra options for the Run button
    if mainfile:
      self._qa_run.setToolTip("Saves this file and compiles the main script "+self._mainfile_base);
      self._qa_run.setText("Compile "+self._mainfile_base);
    else:
      self._qa_run.setToolTip("Saves and compiles this script.");
      self._qa_run.setText("Compile");

  def reload_file (self):
    text = open(self._filename).read();
    # set save icons, etc.
    self._qa_revert.setEnabled(True);
    self._file_disktime = _file_mod_time(self._filename);
    self._document.setPlainText(text);
    self._document.setModified(False);
    self._editor.setReadOnly(not os.access(self._filename,os.W_OK));

  def load_file (self,filename,text=None,readonly=False,mainfile=None):
    """loads editor content.
    filename is filename. text is file contents, if none then file will be re-read.
    readonly is True for readonly mode.
    If mainfile is not None, then this editor is "slaved" to the mainfile. This is the
    case for files included from other modules.
    """
    self.clear_message();
    if not os.access(filename,os.W_OK):
      readonly = True;
    # load text from file if not supplied
    if text is None:
      text = open(filename).read();
    self._filename = filename;
    # sets as as the mainfile or as a submodule of a main file
    self._set_mainfile(mainfile);
    # set save icons, etc.
    self._qa_revert.setEnabled(True);
    self._basename = os.path.basename(filename);
    self._readonly = readonly;
    self._file_disktime = filename and _file_mod_time(filename);
    self._document.setPlainText(text);
    self._document.setModified(False);
    self._editor.setReadOnly(not os.access(self._filename,os.W_OK));
    self._text_modified(False);   # to reset labels
    QApplication.processEvents(QEventLoop.ExcludeUserInputEvents);
    # emit signals
    self.emit(PYSIGNAL("fileLoaded()"),self,filename);
    # if module is a main-level file (i.e. not slaved to another mainfile),
    # pre-import it so that compile-time menus become available
    self._tdlmod = None;
    if not mainfile:
      self.import_content();

  def adjust_editor_font (self):
    pass;

  def has_focus (self,focus):
    if focus:
      TDLEditor.current_editor = self;
    else:
      if TDLEditor.current_editor == self:
        TDLEditor.current_editor = None;

